import pygame
import toolbox as t
import math
import random as r
from NPC import *
import os
#from InnPC import *

DISHES = len(os.listdir('../assets/meals')) - 2

class Building(pygame.sprite.Sprite):
    dimensions = (600, 600)
    def __init__(self, server, x, y, owner):
        '''
        owner must be client channel, not character.

        '''
        pygame.sprite.Sprite.__init__(self, self.gp)
        self.x = x
        self.y = y
        self.type = self.__class__.__qualname__
        self.server = server
        self.level = 1
        self.owner = owner
        self.max_health = self.health = 50
        self.state = 'alive'
        self.dimensions = type(self).dimensions

        self.p = pygame.image.load('../assets/buildings/house.png')
        self.innerrect = self.p.get_rect(center=(x,y))
        self.rect = pygame.Surface((50, 50)).get_rect(midtop=(self.innerrect.midbottom[0], self.innerrect.midbottom[1] + 110))

        

    def post_init(self, rect):
        self.dimensions = rect.size
        self.dim_rect = rect
        self.server.building_blocks.append(self.dim_rect)
        self.server.obs.append(self)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Number of non-broken buildings', str(len(channel.get_buildings()))),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.Out)],
                              'simp':['Heal (5 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'upgradable':False,
                              '4th_info':('Number of non-broken buildings', str(len(channel.get_buildings()))),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}
        

    def update(self):
        

        for p in self.server.players:
            if not p.pending:
                player = p.character
                angle = t.getAngle(self.rect.x, self.rect.y, player.x, player.y)

                screen = pygame.Rect(0, 0, 1000, 650)
                rect = pygame.Rect(0, 0, 1, 1)
                rect.size = self.dim_rect.size
                rect.topleft = (p.character.get_x(self.dim_rect), p.character.get_y(self.dim_rect))
                if screen.colliderect(rect):
                    p.to_send.append({'action':'draw_building',
                        'image':'house',
                        'coords':(player.get_x(self), player.get_y(self)),
                        'health':self.health,
                        'max_health':self.max_health,
                        'angle':angle,
                        'color':self.owner.color,
                        'dimensions':self.dimensions,
                        'type':self.type,
                        'state':self.state,
                        'level':self.level})



    def getHurt(self, damage, attacker):
        
        if self.health > 0 and attacker != self.owner.character:
            self.health -= damage
            if self.health < 1:
                self.state = 'broken'
                if attacker.__class__.__name__ == 'Character':
                    self.owner.message = attacker.channel.username + ' has broken one of your ' + self.type + 's.'
                    attacker.destroyed += 1
                else:
                    self.owner.message = attacker + ' has broken one of your ' + self.type + 's.'
                self.owner.message_count = 150
                self.owner.message_color = (255,0,0)
                self.health = 0

                self.die()
                for p in self.server.players:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    screen.center = p.character.rect.center
                    if screen.colliderect(self.rect):
                        p.to_send.append({'action':'sound', 'sound':'building'})

    def die(self):
        pass

    def isBuilding(self):
        return True


    def Out(self, character):
        
        character.channel.in_window = False
        character.shoot_cooldown = 20
        character.channel.to_send.append({'action':'sound', 'sound':'cw'})


    def heal(self, player):
        if player != self.owner.character:
            player.channel.message = "You can't heal someone else's building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        
        elif player.food > 4:
            if self.health < self.max_health:
                self.health += 10
                if self.health > self.max_health:
                    self.health = self.max_health
                player.channel.message = 'You just healed this building 10 health for 5 food.'
                player.channel.message_count = 120
                player.channel.message_color = (255,205,0)
                player.food -= 5
            else:
                player.channel.message = 'This building is already at full health!'
                player.channel.message_count = 150
                player.channel.message_color = (50,50,255)
        else:
            player.channel.message = "You don't have enough food to heal this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)


    def clear(self, player):
        self.Out(player)
        if player.gold > 1:
            player.channel.message = 'Cleared building for 2 gold'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 2

            self.server.building_blocks.remove(self.dim_rect)
            self.server.obs.remove(self)
            self.kill()
            
        else:
            player.channel.message = "You don't have enough food to clear this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        



class CentralBuilding(Building):
    dimensions = (675, 550)
    def __init__(self, server, x, y, owner):
        Building.__init__(self, server, x, y, owner)
        self.type = 'Central Building'
        self.max_health = self.health = 420
        self.dimensions = type(self).dimensions
        self.info = ('The Central Building is for buying other buildings.', 'If all your buildings are destroyed, you will not respawn.')

        rect = pygame.Rect(0, 0, 675, 550)
        rect.midtop = (x, y-round(self.p.get_height()/2))
        self.post_init(rect)


    def update(self):
        Building.update(self)



    
        
        
    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Number of non-broken buildings', str(len(channel.get_buildings()))),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.buy_central_building), (2, self.buy_fitness_center), (3, self.buy_balloonist), (4, self.buy_farmers_guild), (5, self.buy_miners_guild), (6, self.buy_construction_site), (7, self.buy_restraunt), (8, self.buy_inn), (9, self.Out)],
                              'simp':['Heal (5 food)', 'Portable Central Building (10 gold, 10 food)', 'Fitness Center (50 food)', 'Balloonist (30 gold)', "Farmer's Guild (25 food)", "Miner's Guild (25 gold)", 'Construction Site (60 gold)', 'Restoraunt (80 food)', 'Inn (100 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'upgradable':False,
                              'health':(self.health, self.max_health),
                              'building':self,
                              '4th_info':('Number of non-broken buildings', str(len(channel.get_buildings()))),
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}

    def buy_central_building(self, player):
        if player.food > 9:
            if player.gold > 9:

                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = PortableCentralBuilding
                player.channel.message = 'You just bought a Portable Central Building for 10 gold and 10 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.gold -= 10
                player.food -= 10
            else:
                player.channel.message = "You don't have enough gold to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        
        else:
            if player.gold > 9:
                player.channel.message = "You don't have enough food to buy this!"
            else:
                player.channel.message = "You can't afford this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'gold'

    def buy_fitness_center(self, player):
        if player.food > 49:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = FitnessCenter
            player.channel.message = 'You just bought a Fitness Center for 50 food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= 50
        else:
            player.channel.message = "You don't have enough food to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'

    def buy_farmers_guild(self, player):
        if player.food > 24:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = FarmersGuild
            player.channel.message = "You just bought a Farmer's Guild for 25 food."
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= 25
        else:
            player.channel.message = "You don't have enough food to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'

    def buy_miners_guild(self, player):
        if player.gold > 24:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = MinersGuild
            player.channel.message = "You just bought a Miner's Guild for 25 gold."
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 25
        else:
            player.channel.message = "You don't have enough gold to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'gold'
        


    def buy_balloonist(self, player):
        if player.gold > 29:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = Balloonist
            player.channel.message = 'You just bought a Balloonist for 30 gold.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 30
        else:
            player.channel.message = "You don't have enough gold to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'gold'

    def buy_construction_site(self, player):
        if player.gold > 59:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = ConstructionSite
            player.channel.message = 'You just bought a Construction Site for 60 gold.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 60
        else:
            player.channel.message = "You don't have enough gold to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'gold'
    def buy_restraunt(self, player):
        if player.food > 79:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = Restoraunt
            player.channel.message = 'You just bought a Restoraunt for 80 food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= 80
        else:
            player.channel.message = "You don't have enough food to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'

    def buy_inn(self, player):

        
        yes = True
        for b in player.channel.get_buildings():
            if b.type == 'Inn':
                yes = False
                player.channel.message = 'You already have an Inn! You can only have one at a time.'
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        if yes:
            if player.food > 99:

                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = Inn
                player.channel.message = 'You just bought an Inn for 100 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.food -= 100
            else:
                player.channel.message = "You don't have enough food to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'



    


class PortableCentralBuilding(Building):
    dimensions = (560, 560)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.type = 'Portable Central Building'
        self.max_health = self.health = 110
        self.dimensions = type(self).dimensions
        self.info = ('This building works the same way as the regular Central Building,', 'so you can use it as a portable central building.')

        self.post_init(rect)


    def update(self):
        Building.update(self)



    
        
        
    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Number of non-broken buildings', str(len(channel.get_buildings()))),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.buy_central_building), (2, self.buy_fitness_center), (3, self.buy_balloonist), (4, self.buy_farmers_guild), (5, self.buy_miners_guild), (6, self.buy_construction_site), (7, self.buy_restraunt), (8, self.buy_inn), (9, self.Out)],
                              'simp':['Heal (5 food)', 'Portable Central Building (10 gold, 10 food)', 'Fitness Center (50 food)', 'Balloonist (30 gold)', "Farmer's Guild (25 food)", "Miner's Guild (25 gold)", 'Construction Site (60 gold)', 'Restoraunt (80 food)', 'Inn (100 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'upgradable':False,
                              'health':(self.health, self.max_health),
                              'building':self,
                              '4th_info':('Number of non-broken buildings', str(len(channel.get_buildings()))),
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}

    def buy_central_building(self, player):
        if player.food > 9:
            if player.gold > 9:
    
                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = PortableCentralBuilding
                player.channel.message = 'You just bought a Portable Central Building for 10 gold and 10 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.gold -= 10
                player.food -= 10
            else:
                player.channel.message = "You don't have enough gold to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        
        else:
            if player.gold > 9:
                player.channel.message = "You don't have enough food to buy this!"
            else:
                player.channel.message = "You can't afford this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'gold'
    

    def buy_fitness_center(self, player):
        if player.food > 49:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = FitnessCenter
            player.channel.message = 'You just bought a Fitness Center for 50 food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= 50
        else:
            player.channel.message = "You don't have enough food to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'

    def buy_inn(self, player):

        
        yes = True
        for b in player.channel.get_buildings():
            if b.type == 'Inn':
                yes = False
                player.channel.message = 'You already have an Inn! You can only have one at a time.'
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        if yes:
            if player.food > 99:

                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = Inn
                player.channel.message = 'You just bought an Inn for 100 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.food -= 100
            else:
                player.channel.message = "You don't have enough food to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'

    def buy_farmers_guild(self, player):
        if player.food > 24:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = FarmersGuild
            player.channel.message = "You just bought a Farmer's Guild for 25 food."
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= 25
        else:
            player.channel.message = "You don't have enough food to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'

    def buy_miners_guild(self, player):
        if player.gold > 24:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = MinersGuild
            player.channel.message = "You just bought a Miner's Guild for 25 gold."
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 25
        else:
            player.channel.message = "You don't have enough gold to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'gold'
        


    def buy_balloonist(self, player):
        if player.gold > 29:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = Balloonist
            player.channel.message = 'You just bought a Balloonist for 30 gold.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 30
        else:
            player.channel.message = "You don't have enough gold to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'gold'

    def buy_construction_site(self, player):
        if player.gold > 59:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = ConstructionSite
            player.channel.message = 'You just bought a Construction Site for 60 gold.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 60
        else:
            player.channel.message = "You don't have enough gold to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'gold'
    def buy_restraunt(self, player):
        if player.food > 79:

            player.channel.text = 'Right click to place building'
            player.channel.build_to_place = Restoraunt
            player.channel.message = 'You just bought a Restoraunt for 80 food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= 80
        else:
            player.channel.message = "You don't have enough food to buy this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'


class RunningTrack(Building):
    dimensions = (700, 620)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building increases your speed! Upgrade it to increase it', 'even more. (You can only have one Running Track at a time)')
        self.type = 'Running Track'
        self.health = self.max_health = 180
        self.dimensions = type(self).dimensions
        self.owner.character.speed += 3
        self.owner.character.moving += 3
        self.upgrade_cost = 90

        self.post_init(rect)

    def level_up(self, player):
        if player.food > self.upgrade_cost - 1:
            self.level += 1
            self.owner.character.speed += 3
            self.owner.character.moving += 3
            player.channel.message = 'You upgraded this Running Track to level ' + str(self.level) + ' for ' + str(self.upgrade_cost) + ' food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= self.upgrade_cost
            self.upgrade_cost += 30
            self.health = self.max_health = self.max_health + 40
            
        else:
            player.channel.message = "You don't have enough food to upgrade this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)

        
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            if self.level < 3:
                channel.window = {'text':self.info,
                                  '4th_info':('Speed', str(self.owner.character.speed)),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.level_up), (1, self.heal), (2, self.Out)],
                                  'simp':['Upgrade (' + str(self.upgrade_cost) + ' food)', 'Heal (5 food)', 'Out']}
            else:
                channel.window = {'text':self.info,
                                  '4th_info':('Speed', str(self.owner.character.speed)),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.heal), (1, self.Out)],
                                  'simp':['Heal (5 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              '4th_info':('Speed', str(self.owner.character.speed)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.heal), (2, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}

        
    def getHurt(self, damage, attacker):
        
        if self.health > 0 and attacker != self.owner.character:
            self.health -= damage
            if self.health < 1:
                self.state = 'broken'
                if attacker.__class__.__name__ == 'Character':
                    self.owner.message = attacker.channel.username + ' has broken one of your ' + self.type + 's.'
                    attacker.destroyed += 1
                else:
                    self.owner.message = attacker + ' has broken one of your ' + self.type + 's.'
                self.owner.message_count = 150
                self.owner.message_color = (255,0,0)
                self.health = 0

                self.die()
                for p in self.server.players:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    screen.center = p.character.rect.center
                    if screen.colliderect(self.rect):
                        p.Send({'action':'sound', 'sound':'building'})



                
                self.owner.character.speed = 8
                self.owner.character.moving = 8
                
class Gym(Building):
    dimensions = (700, 620)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building increases your resistance! Upgrade it to increase it', 'even more. (You can only have one Gym at a time)')
        self.type = 'Gym'
        self.health = self.max_health = 180
        self.dimensions = type(self).dimensions
        self.owner.character.strength += 2
        
        self.upgrade_cost = 90

        self.post_init(rect)

    def level_up(self, player):
        if player.food > self.upgrade_cost - 1:
            self.level += 1
            self.owner.character.strength += 2
            
            player.channel.message = 'You upgraded this Gym to level ' + str(self.level) + ' for ' + str(self.upgrade_cost) + ' food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= self.upgrade_cost
            self.upgrade_cost += 30
            self.health = self.max_health = self.max_health + 40
            
        else:
            player.channel.message = "You don't have enough food to upgrade this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)

        
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            if self.level < 3:
                channel.window = {'text':self.info,
                                  '4th_info':('Resistance', str(self.owner.character.strength)),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.level_up), (1, self.heal), (2, self.Out)],
                                  'simp':['Upgrade (' + str(self.upgrade_cost) + ' food)', 'Heal (5 food)', 'Out']}
            else:
                channel.window = {'text':self.info,
                                  '4th_info':('Resistance', str(self.owner.character.strength)),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.heal), (1, self.Out)],
                                  'simp':['Heal (5 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                                  '4th_info':('Resistance', str(self.owner.character.strength)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}
            
    def getHurt(self, damage, attacker):
        
        if self.health > 0 and attacker != self.owner.character:
            self.health -= damage
            if self.health < 1:
                self.state = 'broken'
                if attacker.__class__.__name__ == 'Character':
                    self.owner.message = attacker.channel.username + ' has broken one of your ' + self.type + 's.'
                    attacker.destroyed += 1
                else:
                    self.owner.message = attacker + ' has broken one of your ' + self.type + 's.'
                self.owner.message_count = 150
                self.owner.message_color = (255,0,0)
                self.health = 0

                self.die()
                for p in self.server.players:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    screen.center = p.character.rect.center
                    if screen.colliderect(self.rect):
                        p.Send({'action':'sound', 'sound':'building'})
                self.owner.character.strength = 0
                

class HealthCenter(Building):
    dimensions = (700, 620)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building increases your maximum health! Upgrade it to increase it', 'even more. (You can only have one Health Center at a time)')
        self.type = 'Health Center'
        self.health = self.max_health = 180
        self.dimensions = type(self).dimensions
        self.owner.character.max_health += 30
        self.owner.character.health += 30
        
        self.upgrade_cost = 90

        self.post_init(rect)

    def level_up(self, player):
        if player.food > self.upgrade_cost - 1:
            self.level += 1
            self.owner.character.max_health += 30
            self.owner.character.health += 30
            
            player.channel.message = 'You upgraded this Health Center to level ' + str(self.level) + ' for ' + str(self.upgrade_cost) + ' food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= self.upgrade_cost
            self.upgrade_cost += 30
            self.health = self.max_health = self.max_health + 40
            
        else:
            player.channel.message = "You don't have enough food to upgrade this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)

        
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            if self.level < 3:
                channel.window = {'text':self.info,
                                  '4th_info':('Player health', ('%s/%s' % (self.owner.character.health, self.owner.character.max_health))),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.level_up), (1, self.heal), (2, self.Out)],
                                  'simp':['Upgrade (' + str(self.upgrade_cost) + ' food)', 'Heal (5 food)', 'Out']}
            else:
                channel.window = {'text':self.info,
                                  '4th_info':('Player health', ('%s/%s' % (self.owner.character.health, self.owner.character.max_health))),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.heal), (1, self.Out)],
                                  'simp':['Heal (5 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                                  '4th_info':('Player health', ('%s/%s' % (self.owner.character.health, self.owner.character.max_health))),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}
            
    def getHurt(self, damage, attacker):
        
        if self.health > 0 and attacker != self.owner.character:
            self.health -= damage
            if self.health < 1:
                self.state = 'broken'
                if attacker.__class__.__name__ == 'Character':
                    self.owner.message = attacker.channel.username + ' has broken one of your ' + self.type + 's.'
                    attacker.destroyed += 1
                else:
                    self.owner.message = attacker + ' has broken one of your ' + self.type + 's.'
                self.owner.message_count = 150
                self.owner.message_color = (255,0,0)
                self.health = 0

                self.die()
                for p in self.server.players:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    screen.center = p.character.rect.center
                    if screen.colliderect(self.rect):
                        p.Send({'action':'sound', 'sound':'building'})
                self.owner.character.max_health = 100
                if self.owner.character.health > 100:
                    self.owner.character.health = 100
                




class FitnessCenter(Building):
    dimensions = (585, 600)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building lets you buy the 3 fitness', 'buildings: Running Track, Gym and Health Center.')
        self.type = 'Fitness Center'
        self.health = self.max_health = 140
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def buy_running_track(self, player):
        yes = True
        for b in player.channel.get_buildings():
            if b.type == 'Running Track':
                yes = False
                player.channel.message = 'You already have a Running Track! You can only have one at a time.'
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
                

        if yes:
            if player.food > 74:

                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = RunningTrack
                player.channel.message = 'You just bought a Running Track for 75 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.food -= 75
            else:
                player.channel.message = "You don't have enough food to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'

    def buy_gym(self, player):
        yes = True
        for b in player.channel.get_buildings():
            if b.type == 'Gym':
                yes = False
                player.channel.message = 'You already have a Gym! You can only have one at a time.'
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
                

        if yes:
            if player.food > 74:

                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = Gym
                player.channel.message = 'You just bought a Gym for 75 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.food -= 75
            else:
                player.channel.message = "You don't have enough food to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'

    def buy_health_center(self, player):
        yes = True
        for b in player.channel.get_buildings():
            if b.type == 'Health Center':
                yes = False
                player.channel.message = 'You already have a Health Center! You can only have one at a time.'
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
                

        if yes:
            if player.food > 74:

                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = HealthCenter
                player.channel.message = 'You just bought a Health Center for 75 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.food -= 75
            else:
                player.channel.message = "You don't have enough food to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        self.Out(player)
        player.spence = 'food'


    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Number of fitness buildings', ('%s/3' % len([b for b in self.owner.get_buildings() if b.type in ('Health Center', 'Gym', 'Running Track')]))),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.buy_running_track), (2, self.buy_gym), (3, self.buy_health_center), (4, self.Out)],
                              'simp':['Heal (5 food)', 'Running Track (75 food)', 'Gym (75 food)', 'Health Center (75 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'upgradable':False,
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Number of fitness buildings', ('%s/3' % len([b for b in self.owner.get_buildings() if b.type in ('Health Center', 'Gym', 'Running Track')]))),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}
                              


class Balloonist(Building):
    dimensions = (500, 495)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building lets you upgrade your attack stat.', '')
        self.type = 'Balloonist'
        self.health = self.max_health = 120
        self.dimensions = type(self).dimensions
        self.upgrade_cost = 50


        self.post_init(rect)

    def level_up(self, player):
        if player.gold > self.upgrade_cost - 1:
            self.level += 1
            
            
            player.channel.message = 'You upgraded this Balloonist to level ' + str(self.level) + ' for ' + str(self.upgrade_cost) + ' gold.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= self.upgrade_cost
            self.upgrade_cost += 20
            self.health = self.max_health = self.max_health + 50
            
        else:
            player.channel.message = "You don't have enough gold to upgrade this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Attack Damage', str(self.owner.character.attack)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.level_up), (1, self.heal), (2, self.upgrade_balloon_damage), (3, self.upgrade_balloon_speed), (4, self.upgrade_balloon_knockback), (5, self.Out)],
                              'simp':['Upgrade (' + str(self.upgrade_cost) + ' gold)', 'Heal (5 food)', ' + Balloon Attack (' + str(channel.character.damage_cost) + ' gold)', ' + Balloon Speed (' + str(channel.character.speed_cost) + ' gold)', ' + Balloon Knockback (' + str(channel.character.knockback_cost) + ' gold)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              '4th_info':('Attack Damage', str(self.owner.character.attack)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}

    def upgrade_balloon_speed(self, player):
        if player.gold > player.speed_cost - 1:
            if player.balloon_speed == 18 and self.level == 1:
                player.channel.message = "You have to upgrade the building before upgrading this again."
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
            else:
                
                player.balloon_speed += 2
            
                player.channel.message = "You upgraded your balloons' speed for " + str(self.owner.character.speed_cost) + " gold."
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.gold -= player.speed_cost

                player.speed_cost += 4

            
        else:
            player.channel.message = "You don't have enough gold to upgrade your balloons' speed!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)

    def upgrade_balloon_knockback(self, player):
        if player.gold > player.knockback_cost - 1:
            if player.knockback == 16 and self.level == 1:
                player.channel.message = "You have to upgrade the building before upgrading this again."
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
            else:
                
                player.knockback += 8
            
                player.channel.message = "You upgraded your balloons' knockback for " + str(self.owner.character.knockback_cost) + " gold."
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.gold -= player.knockback_cost

                player.knockback_cost += 4

            
        else:
            player.channel.message = "You don't have enough gold to upgrade your balloons' knockback!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)

    def upgrade_balloon_damage(self, player):
        if player.gold > player.damage_cost - 1:
            if player.attack == 14 and self.level == 1:
                player.channel.message = "You have to upgrade the building before upgrading this again."
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
            else:
                player.attack += 1
            
                player.channel.message = "You upgraded your attack for " + str(self.owner.character.damage_cost) + " gold."
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.gold -= player.damage_cost

                player.damage_cost += 10

            
        else:
            player.channel.message = "You don't have enough gold to upgrade your attack!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)




class FarmersGuild(Building):
    dimensions = (600, 500)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building sends farmers out to collect wheat for you.', '')
        self.type = "Farmer's Guild"
        self.health = self.max_health = 140
        self.dimensions = type(self).dimensions
        self.farmer = Farmer(self)

        self.post_init(rect)

    def getHurt(self, damage, attacker):
        
        if self.health > 0 and attacker != self.owner.character:
            self.health -= damage
            if self.health < 1:
                self.state = 'broken'
                if attacker.__class__.__name__ == 'Character':
                    self.owner.message = attacker.channel.username + ' has broken one of your ' + self.type + 's.'
                    attacker.destroyed += 1
                else:
                    self.owner.message = attacker + ' has broken one of your ' + self.type + 's.'
                self.owner.message_count = 150
                self.owner.message_color = (255,0,0)
                self.health = 0

                self.die()
                for p in self.server.players:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    screen.center = p.character.rect.center
                    if screen.colliderect(self.rect):
                        p.Send({'action':'sound', 'sound':'building'})
                for farmer in self.server.NPCs:
                    if farmer.building == self:
                        farmer.kill()
                
    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Food Production Rate', str(1250 - self.farmer.farm.production)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.Out)],
                              'simp':['Heal (5 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              '4th_info':('Food Production Rate', str(1250 - self.farmer.farm.production)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}

    




class MinersGuild(Building):
    dimensions = (500, 600)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building sends miners out to mine gold for you.', '')
        self.type = "Miner's Guild"
        self.health = self.max_health = 140
        self.dimensions = type(self).dimensions
        self.miner = Miner(self)
        

        self.post_init(rect)

    def getHurt(self, damage, attacker):
        
        if self.health > 0 and attacker != self.owner.character:
            self.health -= damage
            if self.health < 1:
                self.state = 'broken'
                if attacker.__class__.__name__ == 'Character':
                    self.owner.message = attacker.channel.username + ' has broken one of your ' + self.type + 's.'
                    attacker.destroyed += 1
                else:
                    self.owner.message = attacker + ' has broken one of your ' + self.type + 's.'
                self.owner.message_count = 150
                self.owner.message_color = (255,0,0)
                self.health = 0

                self.die()
                for p in self.server.players:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    screen.center = p.character.rect.center
                    if screen.colliderect(self.rect):
                        p.Send({'action':'sound', 'sound':'building'})
                for miner in self.server.NPCs:
                    if miner.building == self:
                        miner.kill()
                

    


    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Gold Discovery Rate', str(1250 - self.miner.farm.production)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.Out)],
                              'simp':['Heal (5 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              '4th_info':('Gold Discovery Rate', str(1250 - self.miner.farm.production)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}



        



def builder(b, p, has_builder):
    if b.isBuilding(object()):
        dimensions = b.dimensions
        if has_builder:
            dimensions = (b.dimensions[0]*0.8, b.dimensions[1]*0.8)
        x, y = p.x, p.y - 140
        image = pygame.image.load('../assets/buildings/house.png')
        width, height = dimensions
        rect = pygame.Rect(0, 0, width, height)
        rect.midtop = (x, y-round(image.get_height()/2))

    else:
        x, y = p.x, p.y - 240
        surf = pygame.Surface((360, 360))
        rect = surf.get_rect(center=(x, y))

    if rect.left < 0 or rect.right > 6000 or rect.top < 0 or rect.bottom > 3900:
        return False, rect
    
    
    
    for obs in p.channel.server.building_blocks:
        if rect.colliderect(obs):
            return False, rect
    return True, rect



class ConstructionSite(Building):
    dimensions = (560, 560)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building lets you buy defences for your village. Crates, Gates, Archery Towers,', 'Robot Factories...')
        self.type = 'Construction Site'
        self.health = self.max_health = 160
        self.dimensions = type(self).dimensions
        self.upgradable = True

        self.post_init(rect)

    def buy_crate(self, player):
        if player.food > 1:
            player.channel.message = 'You bought a crate for 2 food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= 2
            player.crates += 1
        else:
            player.channel.message = "You don't have enough food for this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        
        self.Out(player)

    def buy_gate(self, player):
        if player.gold > 39:
            player.channel.message = 'You bought a gate for 40 gold.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 40
            player.channel.text = 'Press x to place gate'
        else:
            player.channel.message = "You don't have enough gold for this!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        
        self.Out(player)

    def buy_tnt(self, player):
        if player.gold > 99:
            if player.food > 99:
                player.channel.message = 'You bought TNT for 100 gold and 100 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.gold -= 100
                player.food -= 100
                player.channel.text = 'Press x to place TNT'
            else:
                player.channel.message = "You don't have enough food for this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        else:
            if player.food > 99:
                player.channel.message = "You don't have enough gold for this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
            else:
                player.channel.message = "You can't afford this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        
        self.Out(player)

    def buy_archery_tower(self, player):
        if player.food > 19:
            if player.gold > 109:

                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = ArcheryTower
                player.channel.message = 'You just bought an Archery Tower for 110 gold and 20 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.gold -= 110
                player.food -= 20
                player.spence = 'gold'
            else:
                player.channel.message = "You don't have enough gold to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        
        else:
            if player.gold > 109:
                player.channel.message = "You don't have enough food to buy this!"
            else:
                player.channel.message = "You can't afford an Archery Tower!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)

    def level_up(self, player):
        if player.gold > 29:
            self.level = 2
            
            
            player.channel.message = 'You upgraded this Construction Site to level 2 for 30 gold.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 30
            self.upgradable = False
            self.health = self.max_health = 200
            
        else:
            player.channel.message = "You don't have enough gold to upgrade this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)

    def buy_robot_factory(self, player):
        if player.food > 4:
            if player.gold > 114:

                player.channel.text = 'Right click to place building'
                player.channel.build_to_place = RobotFactory
                player.channel.message = 'You just bought a Robot Factory for 115 gold and 5 food.'
                player.channel.message_count = 150
                player.channel.message_color = (255,205,0)
                player.gold -= 115
                player.food -= 5
                player.spence = 'gold'
            else:
                player.channel.message = "You don't have enough gold to buy this!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        
        else:
            if player.gold > 114:
                player.channel.message = "You don't have enough food to buy this!"
            else:
                player.channel.message = "You can't afford this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)
        



    def open_window(self, channel):
        if self.state == 'alive':
            if self.upgradable:
                channel.in_window = True
                channel.window = {'text':self.info,
                              '4th_info':('Upgraded', 'False'),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.level_up), (1, self.heal), (2, self.buy_crate), (3, self.buy_gate), (4, self.buy_tnt), (5, self.Out)],
                              'simp':['Upgrade (30 gold)', 'Heal (5 food)', 'Crate (2 food)', 'Gate (40 gold)', 'Tnt (100 gold, 100 food)', 'Out']}
            else:
                channel.in_window = True
                channel.window = {'text':self.info,
                              '4th_info':('Upgraded', 'True'),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.buy_crate), (2, self.buy_gate), (3, self.buy_tnt), (4, self.buy_archery_tower), (5, self.buy_robot_factory), (6, self.Out)],
                              'simp':['Heal (5 food)', 'Crate (2 food)', 'Gate (40 gold)', 'Tnt (100 gold, 100 food)', 'Archery Tower (110 gold, 20 food)', 'Robot Factory (115 gold, 5 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              '4th_info':('Upgraded', str(self.level == 2)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}




class Restoraunt(Building):
    dimensions = (600, 580)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building lets you buy meals, which heal you when you eat them.', '')
        self.type = 'Restoraunt'
        self.health = self.max_health = 220
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def buy_meal(self, player):

        if player.food > 11:
                if player.meal:
                    player.channel.message = "You already have a meal! Eat it before buying another one."
                    player.channel.message_count = 150
                    player.channel.message_color = (0,0,255)

                else:
                    player.meal = True
                    player.meal_type = r.randrange(DISHES)
                    player.channel.message = "You bought a meal for 12 food."
                    player.channel.message_count = 150
                    player.channel.message_color = (255,205,0)
                    player.food -= 12



            
        else:
            player.channel.message = "You don't have enough food to buy a meal!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)



    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Need meal', ('no' if channel.character.meal else 'yes')),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.buy_meal), (2, self.Out)],
                              'simp':['Heal (5 food)', 'Buy Meal (12 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'upgradable':False,
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Need meal', ('no' if channel.character.meal else 'yes')),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}


class RobotFactory(Building):
    dimensions = (504, 500)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building sends out several little robots who attack', 'other players. It\'s a good way to defend your village!')
        self.type = 'Robot Factory'
        self.health = self.max_health = 120
        self.dimensions = type(self).dimensions

        self.post_init(rect)

        Robot(self)
        Robot(self)
        Robot(self)

        self.num_bots = 3

    def level_up(self, player):
        if player.gold > 39:
            self.level  += 1
            
            
            player.channel.message = 'You upgraded this Robot Factory to level ' + str(self.level) + ' for 40 gold.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.gold -= 40
            self.upgradable = False
            if self.max_health < 320: self.max_health += 50
            self.health = self.max_health

            Robot(self)
            self.num_bots += 1
            
        else:
            player.channel.message = "You don't have enough gold to upgrade this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Robots', str(self.num_bots)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.level_up), (2, self.Out)],
                              'simp':['Heal (5 food)', 'Upgrade (40 gold)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'upgradable':False,
                              'health':(self.health, self.max_health),
                              'building':self,
                              '4th_info':('Robots', str(self.num_bots)),
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}

    def die(self):
        for npc in self.server.NPCs:
            if type(npc).__name__ == 'Robot' and npc.factory == self:
                npc.kill()

from InnPC import *

class Inn(Building):
    dimensions = (550, 490)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building welcomes different NPCs that can trade with you.', '')
        self.type = 'Inn'
        self.health = self.max_health = 240
        self.dimensions = type(self).dimensions
        self.NPC = None
        self.count = r.randint(250, 750)
        self.imposs = []
        

        self.post_init(rect)


    def level_up(self, player):
        if player.food > 99:
            self.level = 2
            
            
            player.channel.message = 'You upgraded this Inn to level 2 for 100 food.'
            player.channel.message_count = 150
            player.channel.message_color = (255,205,0)
            player.food -= 100
            self.upgradable = False
            self.max_health = 340
            self.health = 340
            
        else:
            player.channel.message = "You don't have enough food to upgrade this building!"
            player.channel.message_count = 150
            player.channel.message_color = (255,0,0)
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            if self.NPC:
                if self.level == 1:
                    channel.window = {'text':self.info,
                                          '4th_info':('Hosting', self.NPC.type),
                                          'health':(self.health, self.max_health),
                                          'building':self,
                                          'level':self.level,
                                          'options':[(0, self.heal), (1, self.level_up), (2, self.kick_npc), (3, self.build_house), (4, self.Out)],
                                          'simp':['Heal (5 food)', 'Upgrade (100 food)', 'Kick %s (free)' % (self.NPC.type), 'Build the %s a house (%s gold, %s food)' % (self.NPC.type, self.NPC.cost[0], self.NPC.cost[1]), 'Out']}
                else:
                    channel.window = {'text':self.info,
                                          '4th_info':('Hosting', self.NPC.type),
                                          'health':(self.health, self.max_health),
                                          'building':self,
                                          'level':self.level,
                                          'options':[(0, self.heal), (1, self.kick_npc), (2, self.build_house), (3, self.Out)],
                                          'simp':['Heal (5 food)', 'Kick %s (free)' % (self.NPC.type), 'Build the %s a house (%s gold, %s food)' % (self.NPC.type, self.NPC.cost[0], self.NPC.cost[1]), 'Out']}
            else:
                if self.level == 1:
                    channel.window = {'text':self.info,
                                      '4th_info':('Hosting', 'Nobody'),
                                      'health':(self.health, self.max_health),
                                      'building':self,
                                      'level':self.level,
                                      'options':[(0, self.heal), (1, self.level_up), (2, self.Out)],
                                      'simp':['Heal (5 food)', 'Upgrade (100 food)', 'Out']}
                else:
                    channel.window = {'text':self.info,
                                      '4th_info':('Hosting', 'Nobody'),
                                      'health':(self.health, self.max_health),
                                      'building':self,
                                      'level':self.level,
                                      'options':[(0, self.heal), (1, self.Out)],
                                      'simp':['Heal (5 food)', 'Out']}
        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'upgradable':False,
                              'health':(self.health, self.max_health),
                              'building':self,
                              '4th_info':('Hosting', ('Nobody' if self.NPC is None else self.NPC.type)),
                              'level':self.level,
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 gold)', 'Out']}

    def update(self):
        super().update()
        if self.state != 'broken':
            if self.NPC is not None:
                self.NPC.update()
            else:
                if self.server.event.__class__.__name__ != 'BarbarianRaid':
                    self.count -= 1
                    if self.count == 0:
                        self.NPC = get_innpc(self)
                

    def kick_npc(self, player):
        if player.channel == self.owner:
            name = self.NPC.type
            self.NPC.depart()
            player.channel.message = 'You have kicked the %s.' % name
            player.channel.message_color = (128,0,128)
        else:
            player.channel.message = "This isn't your Inn!"
            player.channel.message_count = 160
            player.channel.message_color = (255,0,0)
        self.Out(player)

    def build_house(self, player):
        if player == self.owner.character:
            if player.food > self.NPC.cost[1]:
                if player.gold > self.NPC.cost[0]:

                    player.channel.text = 'Right click to place building'
                    player.channel.build_to_place = self.NPC.building
                    NPC = self.NPC
                    self.NPC.depart()
                    player.channel.message = 'You just bought the %s a house for %s gold and %s food.' % (NPC.type, NPC.cost[0], NPC.cost[1])
                    player.channel.message_count = 150
                    player.channel.message_color = (255,205,0)
                    player.gold -= NPC.cost[0]
                    player.food -= NPC.cost[1]
                    player.spence = ('gold' if NPC.cost[0] >= NPC.cost[1] else 'food')
                    self.imposs.append(type(NPC))
                else:
                    player.channel.message = "You don't have enough gold to buy this!"
                    player.channel.message_count = 150
                    player.channel.message_color = (255,0,0)
                
            else:
                if player.gold > self.NPC.cost[0]:
                    player.channel.message = "You don't have enough food to buy this!"
                else:
                    player.channel.message = "You can't afford this building!"
                player.channel.message_count = 150
                player.channel.message_color = (255,0,0)
        else:
            player.channel.message = "This isn't your Inn!"
            player.channel.message_count = 160
            player.channel.message_color = (255,0,0)
        self.Out(player)
           
            


                
            



import pygame
import toolbox as t
import random as r
from building import *
from events import *

def get_innpc(inn):
    npcs = [Botanist, Merchant, Rancher, Alchemist]
    if inn.level > 1:
        npcs.extend([Mayor, Repairer, Builder])
    for NPC in inn.imposs:
        if NPC in npcs:
            npcs.remove(NPC)
    try:
        return r.choice(npcs)(inn)
    except:
        return


class InnPC():
    def __init__(self, inn):
        self.inn = inn
        self.type = type(self).__name__.replace('_', ' ').title()
        self.server = inn.server
        self.angle = 0
        self.x = self.inn.x + r.randint(-100, 100)
        self.y = self.inn.y + 130
        self.health = 100
        p = self.inn.owner
        p.message = 'The %s has arrived!' % self.type
        p.message_count = 160
        p.message_color = (128,0,128)
        self.count = r.randint(30*60*1, 30*60*5)
        self.rect = pygame.Surface((50, 50)).get_rect(center=(self.x, self.y))
        self.building = None
        
    @property
    def window(self):
        return {'text':(self.type + ':', 'Hello, there!'),
                'options':(('Out', self.Out),)}


    def Out(self, channel):
        channel.in_window = False
        channel.in_innpc_window = False
        channel.Send({'action':'sound', 'sound':'cw'})

    def update(self):
        self.count -= 1
        if self.count == 0:
            self.depart()
        self.angle = t.getAngle(self.x, self.y, self.inn.owner.character.x, self.inn.owner.character.y)
        for p in self.server.players:
            if not p.pending:
                p.to_send.append({'action':'draw_InnPC',
                                  'type':self.type,
                                  'coords':(p.character.get_x(self), p.character.get_y(self)),
                                  'angle':self.angle,
                                  'color':self.inn.owner.color,
                                  'health':self.health})

    def depart(self, comeback=True):
        self.inn.NPC = None
        p = self.inn.owner
        p.message = 'The %s has departed.' % self.type
        p.message_count = 160
        p.message_color = (128,0,128)
        if comeback:
            self.inn.count = r.randint(300, 900)
            if self.inn.level > 1:
                self.inn.count -= 150


class Botanist(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.cost = (0, 70)
        self.building = BotanistsLab

    @property
    def window(self):
        return {'text':(self.type + ':', 'Hello, there! I sell sapplings, that will grow into any of the trees that the game started with!',
                                         'I also sell spiky bushes. You might have heard of crates, they are the same size except that',
                                         'people can go through the spiky bush, it\'s just that they take damage.'),
                'options':(('Buy a sappling (7 food)', self.sappling), ('Buy a spiky bush (1 food)', self.spiky_bush), ('Out', self.Out))}


    def spiky_bush(self, channel):
        if channel.character.food > 0:
            channel.character.food -= 1
            channel.message = 'You have bought a spiky bush for 1 food.'
            channel.character.spiky_bushes += 1
            channel.message_color = (255,205,0)
            channel.message_count = 150
            channel.character.garden_xp += 1
        else:
            channel.message = 'You don\'t have enough food to buy this!'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

    def sappling(self, channel):
        if channel.character.food > 6:
            channel.character.food -= 7
            channel.message = 'You have bought a sappling for 7 food.'
            channel.text = 'Press c to place sappling'
            channel.message_color = (255,205,0)
            channel.message_count = 150
            channel.character.garden_xp += 20
        else:
            channel.message = 'You don\'t have enough food to buy this!'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

    



class Alchemist(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.cost = (80, 0)
        self.building = AlchemistsLab

    @property
    def window(self):
        return {'text':(self.type + ':', 'Greetings. I can change stone into gold. I might do so for you, for a little money.'),
                'options':(('Increase Gold Discovery Rate by 1 (2 gold)', self.rate1), ('Increase Gold Discovery Rate by 25 (45 gold)', self.rate2), ('Out', self.Out))}

    def rate1(self, channel):
        if channel.character.gold > 1:
            mine = None
            for b in self.inn.owner.get_buildings():
                if b.__class__.__name__ == 'MinersGuild':
                    mine = b.miner.farm
                    break
            if mine and not mine.production == 250:
                channel.character.gold -= 2
                channel.message = 'You have increased your Gold Discovery Rate by 1 for 2 gold.'
                channel.message_color = (255,205,0)
                channel.message_count = 150
                mine.production -= 1
                if mine.production < 250:
                    mine.production = 250
            else:
                channel.message = 'The Gold Discovery Rate is at its maximum!'
                channel.message_color = (0,0,255)
                channel.message_count = 150
            
        else:
            channel.message = 'You don\'t have enough gold!'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

    def rate2(self, channel):
        if channel.character.gold > 44:
            mine = None
            for b in self.inn.owner.get_buildings():
                if b.__class__.__name__ == 'MinersGuild':
                    mine = b.miner.farm
                    break
            if mine and not mine.production == 250:
                channel.character.gold -= 45
                channel.message = 'You have increased your Gold Discovery Rate by 25 for 45 gold.'
                channel.message_color = (255,205,0)
                channel.message_count = 150
                mine.production -= 25
                if mine.production < 250:
                    mine.production = 250
            else:
                channel.message = 'The Gold Discovery Rate is at its maximum!'
                channel.message_color = (0,0,255)
                channel.message_count = 150
            
        else:
            channel.message = 'You don\'t have enough gold!'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

class Rancher(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.cost = (0, 80)
        self.building = Ranch

    @property
    def window(self):
        return {'text':(self.type + ':', 'Howdy, folks! I know better techniques for growing food. I could sell some of those idead, if you want.'),
                'options':(('Increase Food Production Rate by 1 (2 food)', self.rate1), ('Increase Food Production Rate by 25 (45 food)', self.rate2), ('Out', self.Out))}

    def rate1(self, channel):
        if channel.character.food > 1:
            farm = None
            for b in self.inn.owner.get_buildings():
                if b.__class__.__name__ == 'FarmersGuild':
                    farm = b.farmer.farm
                    break
            if farm and not farm.production == 250:
                channel.character.food -= 2
                channel.message = 'You have increased your Food Production Rate by 1 for 2 food.'
                channel.message_color = (255,205,0)
                channel.message_count = 150
                
                farm.production -= 1
                if farm.production < 250:
                    farm.production = 250
            else:
                channel.message = 'The Food Production Rate is at its maximum!'
                channel.message_color = (0,0,255)
                channel.message_count = 150
                
        else:
            channel.message = 'You don\'t have enough food!'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

    def rate2(self, channel):
        if channel.character.food > 44:
            farm = None
            for b in self.inn.owner.get_buildings():
                if b.__class__.__name__ == 'FarmersGuild':
                    farm = b.farmer.farm
                    break
            if farm and not farm.production == 250:
                channel.character.food -= 45
                channel.message = 'You have increased your Food Production Rate by 25 for 45 food.'
                channel.message_color = (255,205,0)
                channel.message_count = 150
                
                farm.production -= 25
                if farm.production < 250:
                    farm.production = 250
            else:
                channel.message = 'The Food Production Rate is at its maximum!'
                channel.message_color = (0,0,255)
                channel.message_count = 150
                
            
        else:
            channel.message = 'You don\'t have enough food!'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)


class Merchant(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.building = Market
        self.cost = (27, 27)

    @property
    def window(self):
        return {'text':(self.type + ':', 'Hello, there! Give me food or gold and I\'ll give you the other.'),
                'options':(('Trade 10 food for 10 gold', self.gold), ('Trade 10 gold for 10 food', self.food), ('Out', self.Out))}

    def gold(self, channel):
        if channel.character.food > 9:
            channel.character.food -= 10
            channel.character.gold += 10
            channel.message = 'You have traded 10 food for 10 gold.'
            channel.message_color = (255,205,0)
            channel.message_count = 150
        else:
            channel.message = 'You don\'t have enough food to trade.'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

    def food(self, channel):
        if channel.character.gold > 9:
            channel.character.gold -= 10
            channel.character.food += 10
            channel.message = 'You have traded 10 gold for 10 food.'
            channel.message_color = (255,205,0)
            channel.message_count = 150
        else:
            channel.message = 'You don\'t have enough gold to trade.'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)



class Mayor(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.building = TownHall
        self.cost = (100, 50)

    @property
    def window(self):
        return {'text':(self.type + ':', 'I\'m in politics. Maybe I\'m useless to you for now, but build me a town hall and you won\'t regret\nit. It\'ll have five hundred health and will repair itself.'),
                'options':(('Out', self.Out),)}

class Retired_barbarian(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.building = RetiredBarbarianOutpost
        self.cost = (0, 20)

    @property
    def window(self):
        return {'text':(self.type + ':', 'Hello. I\'m a barbarian. What\'s that? No, I mean I used to be a barbarian. I see you\'ve had the honor', 'of defeating the enemy tribe of my old one. As a reward, I am selling you some useful barbarian stuff.'),
                'options':(('Out', self.Out), ('Buy a Barbarian Shield (160 food)', self.shield), ('Buy a Barbarian Crossbow (100 food)', self.crossbow))}

    def shield(self, channel):
        if channel.character.has_shield:
            channel.message = 'You already have a shield!'
            channel.message_color = (0,0,255)
            channel.message_count = 150
        elif channel.character.food > 159:
            channel.character.food -= 160
            channel.message = 'You bought a Barbarian Shield for 160 food.'
            channel.message_color = (255,205,0)
            channel.message_count = 150
            channel.character.has_shield = True
        else:
            channel.message = 'You don\'t have enough food to buy this.'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

    def crossbow(self, channel):
        if channel.character.barbshoot_cooldown >= 0:
            channel.message = 'You already have a barbarian crossbow!'
            channel.message_color = (0,0,255)
            channel.message_count = 150
        elif channel.character.food > 99:
            channel.character.food -= 100
            channel.message = 'You bought a barbarian crossbow for 100 food.'
            channel.message_color = (255,205,0)
            channel.message_count = 150
            channel.character.barbshoot_cooldown = 50
        else:
            channel.message = 'You don\'t have enough food to buy this.'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

class RetiredBarbarianOutpost(Building):
    dimensions = (600, 490)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Retired Barbarian.', '')
        self.type = 'Retired Barbarian Outpost'
        self.health = self.max_health = 160
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def crossbow(self, player):
        if player.barbshoot_cooldown >= 0:
            player.channel.message = 'You already have a barbarian crossbow!'
            player.channel.message_color = (0,0,255)
            player.channel.message_count = 150
        elif player.food > 99:
            player.food -= 100
            player.channel.message = 'You bought a barbarian crossbow for 100 food.'
            player.channel.message_color = (255, 205, 0)
            player.channel.message_count = 160
            player.barbshoot_cooldown = 50
        else:
            player.channel.message = 'You don\'t have enough food to buy this.'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    def shield(self, player):
        if player.has_shield:
            player.channel.message = 'You already have a shield!'
            player.channel.message_color = (0,0,255)
            player.channel.message_count = 150
        elif player.food > 159:
            player.food -= 160
            player.channel.message = 'You bought a Barbarian Shield for 160 food.'
            player.channel.message_color = (255, 205, 0)
            player.channel.message_count = 160
            player.has_shield = True
        else:
            player.channel.message = 'You don\'t have enough food to buy this.'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)    




    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Need Shield', str(not channel.character.has_shield)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.shield), (2, self.crossbow), (3, self.Out)],
                              'simp':('Heal', 'Buy a shield (160 gold)', 'Buy a barbarian crossbow (100 food)', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Need Shield', str(not channel.character.has_shield)),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}


class AdventuringCenter(Building):
    dimensions = (510, 400)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Adventurer. Having this building is the best.', '')
        self.type = 'Adventuring Center'
        self.health = self.max_health = 300
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def barbarianRaid(self, player):
        if player.gold > 44:
            player.gold -= 45
            BarbarianRaid(self.server)
            
        else:
            player.channel.message = 'You don\'t have enough gold!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    def barbarianRaidHuge(self, player):
        if player.gold > 99:
            player.gold -= 100
            BarbarianRaid(self.server, 19 + randint(0, 4))
            
        else:
            player.channel.message = 'You don\'t have enough gold!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    def end(self, player):
        for p in self.server.players:
            p.message = player.channel.username + ' has ended the current event.'
            p.message_color = (255, 205, 0)
            p.message_count = 160
        self.server.event = Event(self.server)
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            if self.server.event.__class__ != Event:
                channel.window = {'text':self.info,
                                  '4th_info':('Current Event', self.server.event.__class__.__name__),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.heal), (1, self.end), (2, self.Out)],
                                  'simp':('Heal', 'End Current Event (Free)', 'Out')}
            else:
                channel.window = {'text':self.info,
                                  '4th_info':('Buildings needing repair', str(len([b for b in channel.get_buildings() if b.health < b.max_health]))),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.heal), (1, self.barbarianRaid), (2, self.barbarianRaidHuge), (3, self.Out)],
                                  'simp':('Heal', 'Trigger a Barbarian Raid (45 gold)', 'Trigger a huge Barbarian Raid (100 gold)', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Buildings needing repair', str(len([b for b in channel.get_buildings() if b.health < b.max_health]))),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}

class Adventurer(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.building = AdventuringCenter
        self.cost = (40, 40)

    @property
    def window(self):
        return {'text':(self.type + ':', 'Hello, I\'m the Adventurer. See for youself what I can do!'),
                'options':(('Out', self.Out), ('Trigger a Barbarian Raid (45 gold)', self.barbarianRaid), ('Trigger a huge Babarian Raid (100 gold)', self.barbarianRaidHuge))}

    def barbarianRaid(self, channel):
        if channel.character.gold > 44:
            channel.character.gold -= 45
            BarbarianRaid(self.server)
        else:
            channel.message = 'You don\'t have enough gold to trigger a Barbarian Raid.'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)
        
    def barbarianRaidHuge(self, channel):
        if channel.character.gold > 99:
            channel.character.gold -= 100
            BarbarianRaid(self.server, 19 + randint(0, 4))
        else:
            channel.message = 'You don\'t have enough gold to trigger a huge Barbarian Raid.'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)


class RepairCenter(Building):
    dimensions = (490, 490)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Repairer, so that you can repair your village with ease!', '')
        self.type = 'Repair Center'
        self.health = self.max_health = 290
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def repair(self, player):
        if player.gold > 49:
            player.gold -= 50
            player.channel.message = 'Your village has been repaired for 50 gold.'
            player.channel.message_color = (255, 205, 0)
            player.channel.message_count = 160
            for b in player.channel.get_buildings():
                b.health = b.max_health
            
        else:
            player.channel.message = 'You don\'t have enough gold!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    




    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Buildings needing repair', str(len([b for b in channel.get_buildings() if b.health < b.max_health]))),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.repair), (2, self.Out)],
                              'simp':('Heal', 'Repair Village (50 gold)', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Buildings needing repair', str(len([b for b in channel.get_buildings() if b.health < b.max_health]))),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}

class Repairer(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.building = RepairCenter
        self.cost = (10, 60)

    @property
    def window(self):
        return {'text':(self.type + ':', 'Hello, there! Low on food? I\'d repair everything in your village for just 50 gold!'),
                'options':(('Repair Village (50 gold)', self.repair), ('Out', self.Out))}

    def repair(self, channel):
        if channel.character.gold > 49:
            channel.character.gold -= 50
            channel.message = 'You repaired your entire village for 50 gold.'
            channel.message_color = (255,205,0)
            channel.message_count = 150
            for b in channel.get_buildings():
                b.health = b.max_health
        else:
            channel.message = 'You don\'t have enough gold to repair your village.'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)

class Repairer(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.building = RepairCenter
        self.cost = (10, 60)

    @property
    def window(self):
        return {'text':(self.type + ':', 'Hello, there! Low on food? I\'d repair everything in your village for just 50 gold!'),
                'options':(('Repair Village (50 gold)', self.repair), ('Out', self.Out))}

    def repair(self, channel):
        if channel.character.gold > 49:
            channel.character.gold -= 50
            channel.message = 'You repaired your entire village for 50 gold.'
            channel.message_color = (255,205,0)
            channel.message_count = 150
            for b in channel.get_buildings():
                b.health = b.max_health
        else:
            channel.message = 'You don\'t have enough gold to repair your village.'
            channel.message_color = (255,0,0)
            channel.message_count = 150
        self.Out(channel)


class Builder(InnPC):
    def __init__(self, inn):
        super().__init__(inn)
        self.building = Builders
        self.cost = (30, 0)

    @property
    def window(self):
        base = {'text':(self.type + ':', 'As long as I\'m at the Inn, all the buildings you buy are smaller!')}
        if self.inn.server.fallen:
            base['options'] = (('Out', self.Out),)
        else:
            base['options'] = (('Add a minute till the Walls fall (100 food)', self.minute), ('Out', self.Out))
        return base
                

    def minute(self, channel):
        if not self.inn.server.fallen:
            if channel.character.food > 99:
                channel.character.food -= 100
                channel.message = 'You added a minute till the Walls fall.'
                channel.message_color = (255,205,0)
                channel.message_count = 150
                self.inn.server.upwall.count += 30*60
                self.inn.server.leftwall.count += 30*60
                
            else:
                channel.message = 'You don\'t have enough food to do this.'
                channel.message_color = (255,0,0)
                channel.message_count = 150
        else:
            channel.message = 'The Walls have already fallen!'
            channel.message_color = (0,0,255)
            channel.message_count = 150
        self.Out(channel)

class Builders(Building):
    dimensions = (490 * 0.8, 490 * 0.8)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Builder. As long as this buildings isn\'t broken, all the buildings you buy will be smaller.', '')
        self.type = 'Builder\'s'
        self.health = self.max_health = 300
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def minute(self, player):
        if not self.server.fallen:
            if player.food > 90:
                player.food -= 90
                player.channel.message = 'You added a minute till the Walls fall.'
                player.channel.message_color = (255, 205, 0)
                player.channel.message_count = 160
                self.server.upwall.count += 30*60
                self.server.leftwall.count += 30*60
                
            else:
                player.channel.message = 'You don\'t have enough food to do this.'
                player.channel.message_color = (255,0,0)
                player.channel.message_count = 150
        else:
            player.channel.message = 'The Walls have already fallen!'
            player.channel.message_color = (0,0,255)
            player.channel.message_count = 150
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            if self.server.fallen:
                channel.window = {'text':self.info,
                                  '4th_info':('Walls Fallen', 'True'),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.heal), (1, self.Out)],
                                  'simp':('Heal', 'Out')}
            else:
                channel.window = {'text':self.info,
                                  '4th_info':('Time till the Walls fall', self.server.getTime()),
                                  'health':(self.health, self.max_health),
                                  'building':self,
                                  'level':self.level,
                                  'options':[(0, self.heal), (1, self.minute), (2, self.Out)],
                                  'simp':('Heal', 'Add a minute till the Walls fall (90 food)', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Walls Fallen', str(self.server.fallen)),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}

class AdvancedBalloonistBuilding(Building):
    dimensions = (490, 490)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Advanced Balloonist NPC, so that you can upgrade your shot speed easily.', '')
        self.type = 'Advanced Balloonist'
        self.health = self.max_health = 300
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def speedgun(self, player):
        if player.shot_speed != 1:
            if player.gold > 279:
                player.gold -= 280
                player.channel.message = 'You increased your shoot speed for 280 gold.'
                player.channel.message_color = (255, 205, 0)
                player.channel.message_count = 160
                player.shot_speed -= 1
                
            else:
                player.channel.message = 'You don\'t have enough gold to do this.'
                player.channel.message_color = (255,0,0)
                player.channel.message_count = 150
        else:
            player.channel.message = 'Your balloon gun is already at its max! (Nobody\'s gonna stop you now)'
            player.channel.message_color = (0,0,255)
            player.channel.message_count = 150
        self.Out(player)

    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Shot Speed', str(16-channel.character.shot_speed)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.speedgun), (2, self.Out)],
                              'simp':('Heal', '+ Shot Speed (280 gold)', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Shot Speed', str(16-channel.character.shot_speed)),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}


class TownHall(Building):
    dimensions = (560, 540)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building has 500 HP! It\'ll also repair itself fully if it hasn\'t been attacked in a minute.', '')
        self.type = 'Town Hall'
        self.health = self.max_health = 500
        self.dimensions = type(self).dimensions
        self.count = 0

        self.post_init(rect)

    def update(self):
        super().update()
        if self.count:
            self.count -= 1
            if not self.count:
                self.health = self.max_health

    def getHurt(self, damage, attacker):
        super().getHurt(damage, attacker)
        self.count = 1800


class AlchemistsLab(Building):
    dimensions = (490, 490)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Alchemist, so that you can always trade with it.', '')
        self.type = 'Alchemist\'s Lab'
        self.health = self.max_health = 220
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def rate1(self, player):
        if player.gold > 1:
            mine = None
            for b in self.owner.get_buildings():
                if b.__class__.__name__ == 'MinersGuild':
                    mine = b.miner.farm
                    break
            if mine and not mine.production == 250:
                player.gold -= 2
                player.channel.message = 'You have increased your Gold Discovery Rate by 1 for 2 gold.'
                player.channel.message_color = (255,205,0)
                player.channel.message_count = 150
                
                mine.production -= 1
                if mine.production < 250:
                    mine.production = 250
            else:
                player.channel.message = 'The Gold Discovery Rate is at its maximum!'
                player.channel.message_color = (0,0,255)
                player.channel.message_count = 150
        else:
            player.channel.message = 'You don\'t have enough gold!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    def rate2(self, player):
        if player.gold > 44:
            mine = None
            for b in self.owner.get_buildings():
                if b.__class__.__name__ == 'MinersGuild':
                    mine = b.miner.farm
                    break
            if mine and not mine.production == 250:
                player.gold -= 45
                player.channel.message = 'You have increased your Gold Discovery Rate by 25 for 45 gold.'
                player.channel.message_color = (255,205,0)
                player.channel.message_count = 150
                
                mine.production -= 25
                if mine.production < 250:
                    mine.production = 250
            else:
                player.channel.message = 'The Gold Discovery Rate is at its maximum!'
                player.channel.message_color = (0,0,255)
                player.channel.message_count = 150
            
        else:
            player.channel.message = 'You don\'t have enough gold!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    




    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Gold', str(channel.character.gold)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.rate1), (2, self.rate2), (3, self.Out)],
                              'simp':('Heal', 'Increase Gold Discovery Rate by 1 (2 gold)', 'Increase Gold Discovery Rate by 25 (45 gold)', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Gold', str(channel.character.gold)),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}

class Ranch(Building):
    dimensions = (490, 490)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Rancher, so that you can always trade with it.', '')
        self.type = 'Ranch'
        self.health = self.max_health = 220
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def rate1(self, player):
        if player.food > 1:
            farm = None
            for b in self.owner.get_buildings():
                if b.__class__.__name__ == 'FarmersGuild':
                    farm = b.farmer.farm
                    break
            if farm and not farm.production == 250:
                player.food -= 2
                player.channel.message = 'You have increased your Food Production Rate by 1 for 2 food.'
                player.channel.message_color = (255,205,0)
                player.channel.message_count = 150
                
                farm.production -= 1
                if farm.production < 250:
                    farm.production = 250
            else:
                player.channel.message = 'The Food Production Rate is at its maximum!'
                player.channel.message_color = (0,0,255)
                player.channel.message_count = 150
        else:
            player.channel.message = 'You don\'t have enough food!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    def rate2(self, player):
        if player.food > 44:
            farm = None
            for b in self.owner.get_buildings():
                if b.__class__.__name__ == 'FarmersGuild':
                    farm = b.farmer.farm
                    break
            if farm and not farm.production == 250:
                player.food -= 45
                player.channel.message = 'You have increased your Food Production Rate by 25 for 45 food.'
                player.channel.message_color = (255,205,0)
                player.channel.message_count = 150
                
                farm.production -= 25
                if farm.production < 250:
                    farm.production = 250
            else:
                player.channel.message = 'The Food Production Rate is at its maximum!'
                player.channel.message_color = (0,0,255)
                player.channel.message_count = 150
            
        else:
            player.channel.message = 'You don\'t have enough food!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)



    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Food', str(channel.character.food)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.rate1), (2, self.rate2), (3, self.Out)],
                              'simp':('Heal', 'Increase Food Production Rate by 1 (2 food)', 'Increase Food Production Rate by 25 (45 food)', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Gold', str(channel.character.gold)),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}


class BotanistsLab(Building):
    dimensions = (490, 490)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Botanist, so that you can always buy from it.', '')
        self.type = 'Botanist\'s Lab'
        self.health = self.max_health = 220
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def spiky_bush(self, player):
        if player.food > 0:
            player.food -= 1
            player.channel.message = 'You have bought a spiky bush for 1 food.'
            player.spiky_bushes += 1
            player.channel.message_color = (255,205,0)
            player.channel.message_count = 150
            player.garden_xp += 1
        else:
            player.channel.message = 'You don\'t have enough food to buy this!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    def sappling(self, player):
        if player.food > 6:
            player.food -= 7
            player.channel.message = 'You have bought a sappling for 7 food.'
            player.channel.text = 'Press c to place sappling'
            player.channel.message_color = (255,205,0)
            player.channel.message_count = 150
            player.garden_xp += 20
        else:
            player.channel.message = 'You don\'t have enough food to buy this!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    def vine(self, player):
        if player.food > 14:
            player.food -= 15
            player.channel.message = 'You have bought an invasive vine for 15 food.'
            player.channel.text = 'Press c to place invasive vine'
            player.channel.message_color = (255,205,0)
            player.channel.message_count = 150
        else:
            player.channel.message = 'You don\'t have enough food to buy this!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)




    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Gardening XP', str(channel.character.garden_xp)),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.sappling), (2, self.spiky_bush), (3, self.Out)],
                              'simp':('Heal', 'Buy a sappling (7 food)', 'Buy a spiky bush (1 food)', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Gardening XP', str(channel.character.garden_xp)),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}

class Market(Building):
    dimensions = (490, 490)
    def __init__(self, server, x, y, owner, rect):
        Building.__init__(self, server, x, y, owner)
        self.info = ('This building is for the Merchant, so that you can always trade', 'gold for food and food for gold.')
        self.type = 'Market'
        self.health = self.max_health = 220
        self.dimensions = type(self).dimensions

        self.post_init(rect)

    def tradegold(self, player):
        if player.food > 9:
            player.food -= 10
            player.gold += 10
            player.channel.message = 'You have traded 10 food for 10 gold.'
            player.channel.message_color = (255,205,0)
            player.channel.message_count = 150
        else:
            player.channel.message = 'You don\'t have enough food to trade!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)
    def tradefood(self, player):
        if player.gold > 9:
            player.gold -= 10
            player.food += 10
            player.channel.message = 'You have traded 10 gold for 10 food.'
            player.channel.message_color = (255,205,0)
            player.channel.message_count = 150
        else:
            player.channel.message = 'You don\'t have enough gold to trade!'
            player.channel.message_color = (255,0,0)
            player.channel.message_count = 150
        self.Out(player)

    
    def open_window(self, channel):
        if self.state == 'alive':
            channel.in_window = True
            channel.window = {'text':self.info,
                              '4th_info':('Resources', str(self.owner.character.gold) + ' gold, ' + str(self.owner.character.food) + ' food'),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              'options':[(0, self.heal), (1, self.tradegold), (2, self.tradefood), (3, self.Out)],
                              'simp':('Heal', 'Trade 10 food for 10 gold', 'Trade 10 gold for 10 food', 'Out')}

        elif self.state == 'broken':
            channel.in_window = True
            channel.window = {'text':('This building is broken.', ''),
                              'health':(self.health, self.max_health),
                              'building':self,
                              'level':self.level,
                              '4th_info':('Resources', str(self.owner.character.gold) + ' gold, ' + str(self.owner.character.food) + ' food'),
                              'options':[(0, self.clear), (1, self.Out)],
                              'simp':['Clear (2 food)', 'Out']}

    



import pygame
import toolbox as t
from balloon import Balloon, Bolt
from obstacle import *
from building import *
from animations import *
from NPC import ArcheryTower

class Character():



    def __init__(self, channel, x, y):

        self.channel = channel
        self.image = pygame.Surface((50, 50))
        self.x = x
        self.y = y
        self.x_flip = 500-x
        self.y_flip = 325-y
        self.angle = 0
        self.speed = self.moving = 8
        
        self.health = self.max_health = 100
        self.hurt_count = 1
        self.dead = False

        if self.channel.username == 'ModestNoob' or self.channel.username == 'f':
            self.gold = 100000
            self.food = 100000
        else:
            self.gold = 0
            self.food = 0
        
        self.strength = 0
        self.obs = []
        self.rect = self.image.get_rect(center=(self.x, self.y))

        self.shoot_cooldown = 0
        self.barbshoot_cooldown = -1
        self.shot_speed = 15
        self.crate_cooldown = 0

        self.respawn_count = 0
        self.meal = False
        self.meal_type = 0

        self.crates = 0
        self.spiky_bushes = 0
        self.spence = 'gold'

        self.garden_xp = 0

        self.shield_count = 0
        self.shield_angle = 0
        self.has_shield = False


        ### Stat Side ###

        
        self.attack = 10
        self.damage_cost = 30
        
        self.balloon_speed = 16
        self.speed_cost = 10
        
        self.knockback = 10
        self.knockback_cost = 10


        ### Info Side ###

        self.kills = 0
        self.destroyed = 0
        self.deaths = 0
        self.eliminations = 0

    @property
    def have_builder(self):
        true = False
        for inn in self.channel.get_buildings():
            if inn.__class__.__name__ == 'Inn':
                if inn.NPC.__class__.__name__ == 'Builder':
                    true = True
                    break
            if inn.type == 'Builder\'s':
                true = True
                break
        return true
        

    def get_x(self, item):

        try:
            return item.x + self.x_flip
        except:
            return item + self.x_flip
        

    def get_y(self, item):

        try:
            return item.y + self.y_flip
        except:
            return item + self.y_flip

    def move_x(self, a, type_=None):
        
        self.x += a
        self.x_flip -= a

        self.rect = self.image.get_rect(center=(self.x, self.y))


        if not self.dead:
            for item in self.obs:
                thr = False
                if item.__class__ == Gate and item.owner == self:
                    thr = True
                if self.rect.colliderect(item.innerrect) and not thr:
                    self.x -= a
                    self.x_flip += a

                
                    

    def move_y(self, a, type_=None):
        self.y += a
        self.y_flip -= a

        self.rect = self.image.get_rect(center=(self.x, self.y))

        if not self.dead:
            for item in self.obs:
                thr = False
                if item.__class__ == Gate and item.owner == self:
                    thr = True
                if self.rect.colliderect(item.innerrect) and not thr:
                    self.y -= a
                    self.y_flip += a
            for item in self.channel.server.bushes:
                if self.rect.colliderect(item.innerrect) and type_ != 'hurt':
                    if item.__class__.__name__ == 'SpikyBush':
                        self.getHurt(1, 'a spiky bush', self.angle + 180, 4, msg='<Victim> fell in <Attacker>.')
                    else:
                        self.getHurt(0, '', self.angle + 180, 4)
             
                    
        
    def move(self, angle):
        x, y = t.getDir(angle, self.get_speed())
        self.move_x(round(x))
        self.move_y(round(y))

    def update(self):
        
        if self.channel.build_to_place:

            self.channel.to_send.append({'action':'preview', 'dimensions':((self.channel.build_to_place.dimensions[0]*0.8, self.channel.build_to_place.dimensions[1]*0.8) if self.have_builder else self.channel.build_to_place.dimensions), 'ArcheryTower?':self.channel.build_to_place==ArcheryTower})
        
        self.rect = self.image.get_rect(center=(self.x, self.y))
        if not self.dead:
            for item in self.obs:
                if item.__class__ == Crate and self.rect.colliderect(item.innerrect):
                    x, y = t.getDir(self.angle + 180, 40)
                    self.x += x
                    self.x_flip -= x
                    self.y += y
                    self.y_flip -= y
                else:
                    thr = False
                    if item.__class__ == Gate and item.owner == self:
                        thr = True
                    if self.rect.colliderect(item.innerrect) and not thr:
                        self.suffocate()


        if self.shoot_cooldown > 0:
            self.shoot_cooldown -= 1
        if self.crate_cooldown > 0:
            self.crate_cooldown -= 1
        if self.hurt_count > 1:
            self.hurt_count -= 1
        if self.shield_count:
            self.shield_count -= 1
        if self.barbshoot_cooldown:
            self.barbshoot_cooldown -= 1

        if not self.barbshoot_cooldown:
            self.barbshoot()

        if self.respawn_count > 0:
            self.respawn_count -= 1
            if self.respawn_count == 0:
                self.respawn()
        
        self.obs = self.channel.server.obs
        
        
        if not self.dead:
            for p in self.channel.server.players:
            
                if p != self.channel:
                    if not p.pending:
                        
                    
                        p.to_send.append({'action':'draw_player',
                                'hurt':int(bool(self.hurt_count - 1)),
                                'coords':(p.character.get_x(self), p.character.get_y(self)),
                                'angle':self.angle,
                                'color':self.channel.color,
                                'username':self.channel.username,
                                'health':self.health,
                                'max_health':self.max_health,
                                'skin':self.channel.skin,
                                'shield':(self.shield_angle if self.shield_count else None)})
                else:
                    p.to_send.append({'action':'draw_player',
                            'hurt':int(bool(self.hurt_count - 1)),
                            'coords':(p.character.get_x(self), p.character.get_y(self)),
                            'angle':self.angle,
                            'color':self.channel.color,
                            'username':self.channel.username,
                            'health':self.health,
                            'gold':self.gold,
                            'food':self.food,
                            'max_health':self.max_health,
                            'skin':self.channel.skin,
                            'shield':(self.shield_angle if self.shield_count else None)})
                    
                    p.to_send.append({'action':'meal',
                            'food?':self.meal,
                                      'type':self.meal_type})
                    
                    if self.crates:
                        p.to_send.append({'action':'crates',
                            'crates':self.crates})
                    if self.spiky_bushes:
                        p.to_send.append({'action':'spiky_bushes',
                            'num':self.spiky_bushes})

    def barbshoot(self):
        
        dire = 650
        player = None
        for p in self.channel.server.players:
            if t.getDist(self.x, self.y, p.character.x, p.character.y) < dire and p != self.channel:
                player = p.character
                dire = t.getDist(self.x, self.y, p.character.x, p.character.y)
        if player:
            self.angle = t.getAngle(self.x, self.y, player.x, player.y)
            Bolt(self)
            self.barbshoot_cooldown = 50
        

    def HandleInput(self, keys, mouse):
        if not self.channel.in_window:
            self.angle = t.getAngle(500, 325, mouse[0], mouse[1])

            if keys[pygame.K_a]:
                self.channel.achievement('You pressed the A button!')
            if keys[pygame.K_8]:
                self.channel.achievement('You pressed the 8 button!')
                
            if keys[pygame.K_SPACE]:
                self.move(self.angle)

            if keys[pygame.K_c] and not self.dead:
                if self.channel.text == 'Press c to place sappling':
                    Sappling(self.channel.server, self.x, self.y)
                    self.channel.text = ''
                    self.crate_cooldown = 30
                elif self.channel.text == 'Press c to place invasive vine':
                    Vine(self, self.x, self.y + 60)
                    self.channel.text = ''
                    self.crate_cooldown = 30
                elif self.crate_cooldown == 0 and self.spiky_bushes > 0:
                    x, y = self.x, self.y
                    move = t.getDir(self.angle, 80)
                    x += move[0]
                    y += move[1]
                        
                    SpikyBush(self, x, y)
                    self.crate_cooldown = 30
                    self.spiky_bushes -= 1

            if keys[pygame.K_x]:
                if self.channel.text == 'Press x to place gate':
                    self.channel.text = ''
                    x, y = self.x, self.y
                    move = t.getDir(self.angle, 100)
                    x += move[0]
                    y += move[1]
                    rotated = False
                    if 45 < self.angle < 135 or -45 > self.angle > -135:
                        rotated = True
                    
                    Gate(self, x, y, rotated)
                    self.crate_cooldown = 40

                elif self.channel.text == 'Press x to place TNT':
                    self.channel.text = ''
                    x, y = self.x, self.y
                    move = t.getDir(self.angle, 80)
                    x += move[0]
                    y += move[1]
                    
                    TNT(self, x, y)
                    self.crate_cooldown = 40
                    
                elif not self.dead and self.crate_cooldown == 0 and self.crates > 0:
                    x, y = self.x, self.y
                    move = t.getDir(self.angle, 80)
                    x += move[0]
                    y += move[1]
                    
                    Crate(self, x, y)
                    self.crate_cooldown = 30
                    self.crates -= 1

            if keys[pygame.K_DELETE] and self.channel.build_to_place != None:
                self.channel.build_to_place = None
                self.channel.text = ''
                if self.spence == 'gold':
                    self.channel.message = 'You deleted the building, you gain 10 gold.'
                    self.gold += 10
                else:
                    self.channel.message = 'You deleted the building, you gain 10 food.'
                    self.food += 10
                self.channel.message_count = 150
                self.channel.message_color = (255,100,0)

            
            if mouse[2] and not self.dead:

                not_shoot = False
                if self.dead == False:
                    for farm in self.channel.server.resources:
                        if farm.HandleInput(mouse, self):
                            not_shoot = True

                if not not_shoot:
                    self.shoot()

        if self.channel.in_window and mouse[2] and not self.dead and not self.channel.in_innpc_window:
            not_farm = True
            not_shoot = True
            for option in self.channel.window['options']:
                
                x = 200
                y = 200 + option[0] * 40
                rect = pygame.Rect(x, y, 600, 50)

                if rect.collidepoint((mouse[0], mouse[1])):
                    option[1](self)
                    if self.channel.message == 'The Gold Discovery Rate is at its maximum!' and len([b for b in self.channel.get_buildings() if b.type == 'Miner\'s Guild']) == 0:
                        self.channel.message = 'Get a Miner\'s Guild first.'
                    if self.channel.message == 'The Food Production Rate is at its maximum!' and len([b for b in self.channel.get_buildings() if b.type == 'Farmer\'s Guild']) == 0:
                        self.channel.message = 'Get a Farmer\'s Guild first.'
                    break
                
        if self.channel.in_window and mouse[2] and not self.dead and self.channel.in_innpc_window:
            for i, option in enumerate(self.channel.window['options']):
                
                x = 200
                y = 200 + i * 40
                rect = pygame.Rect(x, y, 600, 50)

                if rect.collidepoint((mouse[0], mouse[1])):
                    option[1](self.channel)
                    if self.channel.message == 'The Gold Discovery Rate is at its maximum!' and len([b for b in self.channel.get_buildings() if b.type == 'Miner\'s Guild']) == 0:
                        self.channel.message = 'Get a Miner\'s Guild first.'
                    if self.channel.message == 'The Food Production Rate is at its maximum!' and len([b for b in self.channel.get_buildings() if b.type == 'Farmer\'s Guild']) == 0:
                        self.channel.message = 'Get a Farmer\'s Guild first.'
                    break


        if mouse[4] and not self.dead:
            
            if self.channel.build_to_place != None:
                result = builder(self.channel.build_to_place, self, self.have_builder)
                to_build = result[0]
                if to_build:
                    b = self.channel.build_to_place(self.channel.server, self.x, self.y - 140, self.channel, result[1])

                    self.channel.text = ''
                    self.channel.build_to_place = None
                    try:
                        self.channel.message = 'You have placed a ' + b.type + '.'
                    except:
                        self.channel.message = 'You have placed an Archery Tower.'
                    self.channel.message_count = 150
                    self.channel.message_color = (255,205,0)
                else:
                    
                    self.channel.message = 'You cannot place this building here.'
                    self.channel.message_count = 100
                    self.channel.message_color = (255,0,0)
            elif not self.channel.in_window and not self.channel.in_innpc_window:
                for b in self.channel.server.buildings:
                    rect = b.p.get_rect(center=(self.get_x(b), self.get_y(b)))
                    if rect.collidepoint((mouse[0], mouse[1])):
                        b.open_window(self.channel)
                        self.channel.to_send.append({'action':'sound', 'sound':'ow'})

                    else:
                        rect = b.rect.copy()
                        rect.x, rect.y = self.get_x(rect), self.get_y(rect)
                        if rect.collidepoint((mouse[0], mouse[1])):
                            b.open_window(self.channel)
                            self.channel.to_send.append({'action':'sound', 'sound':'ow'})

                    if b.__class__ == Inn and b.NPC != None and b.owner == self.channel:
                        rect = pygame.Rect(0, 0, 0, 0)
                        rect.topleft = self.get_x(b.NPC.rect), self.get_y(b.NPC.rect)
                        rect.size = b.NPC.rect.size
                        if rect.collidepoint((mouse[0], mouse[1])):
                            self.channel.in_window = True
                            self.channel.in_innpc_window = True
                            self.channel.window = b.NPC.window
                            self.channel.to_send.append({'action':'sound', 'sound':'ow'})



    def eat(self):
        if not self.dead:
            self.meal = False
            self.health = self.max_health
            self.channel.message = 'You ate your meal, your health bar is now full again.'
            self.channel.message_count = 150
            self.channel.message_color = (255,205,0)
            
        
    def getHurt(self, damage, attacker, angle, knockback, msg='<Attacker> killed <Victim>.'):
        if randint(0, 2) == 0 and self.has_shield:
            self.shield_angle = angle + 180
            self.shield_count = 20
            for p in self.channel.server.players:
                if not p.pending:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    rect = pygame.Rect(0, 0, 1, 1)
                    rect.size = self.rect.size
                    rect.topleft = (p.character.get_x(self), p.character.get_y(self))
                    if screen.colliderect(rect):
                        p.to_send.append({'action':'sound','sound':'bump'})
            return 'repelled'
        final_damage = (damage - self.strength)
        if final_damage < 0:
            final_damage = 0
        self.health -= final_damage
        x, y = t.getDir(angle, knockback)
        self.move_x(x, 'hurt')
        self.move_y(y, 'hurt')
        self.hurt_count = 8
        if not self.health > 0:
            Explosion(self)
            for p in self.channel.server.players:
                screen = pygame.Rect(0, 0, 1000, 650)
                screen.center = p.character.rect.center
                if screen.colliderect(self.rect):
                    p.to_send.append({'action':'sound', 'sound':'die'})
            self.channel.text = ''
            self.channel.build_to_place = None
            self.dead = True
            
            self.deaths += 1
            self.crates = 0
            self.meal = False
            self.moving = 16
            if not isinstance(attacker, str):
                attacker.kills += 1
                newdict = self.channel.server.database[attacker.channel.username]
                newdict['kills'] += 1
                self.channel.server.database[attacker.channel.username] = newdict
                username = attacker.channel.username
            else:
                username = attacker

            msg = msg.replace('<Attacker>', username).replace('<Victim>', self.channel.username)
            for channel in self.channel.server.players:
                channel.message = msg
                channel.message_count = 150
                channel.message_color = (255,205,0)
                if channel == self.channel:
                    channel.message_color = (255,0,0)
            if len(self.channel.get_buildings()) != 0:
                self.respawn_count = 200
            else:
                if not isinstance(attacker, str):
                    attacker.eliminations += 1
                guys = []
                for channel in self.channel.server.players:
                    channel.message = username + ' has eliminated ' + self.channel.username + ' from the game.'
                    channel.message_count = 150
                    channel.message_color = (255,205,0)
                    if channel == self.channel:
                        channel.message = username + ' has eliminated you from the game!'
                        channel.message_color = (255,0,0)
                    if channel.character.dead == False:
                        guys.append(channel)
                    elif channel.character.respawn_count > 0:
                        guys.append(channel)
                if len(guys) == 1:
                    self.channel.server.terminate(guys[0])

        self.channel.in_window = False
        self.channel.window = None

    def suffocate(self):
        Explosion(self)
        for p in self.channel.server.players:
            screen = pygame.Rect(0, 0, 1000, 650)
            screen.center = p.character.rect.center
            if screen.colliderect(self.rect):
                p.to_send.append({'action':'sound', 'sound':'die'})
        self.channel.text = ''
        self.channel.build_to_place = None
        self.dead = True
            
        self.deaths += 1
        self.crates = 0
        self.meal = False
        self.moving = 16

        for channel in self.channel.server.players:
            channel.message = self.channel.username + ' suffocated.'
            channel.message_count = 150
            channel.message_color = (255,205,0)
            if channel == self.channel:
                channel.message_color = (255,0,0)

        if len(self.channel.get_buildings()) != 0:
            self.respawn_count = 200
        else:
            guys = []
            for channel in self.channel.server.players:
                channel.message = self.channel.username + ' has been eliminated from the game by suffocation.'
                channel.message_count = 150
                channel.message_color = (255,205,0)
                if channel == self.channel:
                    channel.message = 'You suffocated and were eliminated from the game!'
                    channel.message_color = (255,0,0)
                if channel.character.dead == False:
                    guys.append(channel)
                elif channel.character.respawn_count > 0:
                    guys.append(channel)
            if len(guys) == 1:
                self.channel.server.terminate(guys[0])



    def shoot(self):
        if self.shoot_cooldown == 0:
            Balloon(self.channel.server, self)
            self.shoot_cooldown = self.shot_speed
            for p in self.channel.server.players:
                if not p.pending:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    screen.center = p.character.rect.center
                    if screen.colliderect(self.rect):
                        p.to_send.append({'action':'sound', 'sound':'shot'})


    def respawn(self):
        self.dead = False
        self.x, self.y = self.channel.server.ST_COORDS[self.channel.loc_number]
        self.x_flip = 500-self.x
        self.y_flip = 325-self.y
        self.moving = self.speed
        self.channel.message = 'You respawned. You respawn as long as you have buildings alive.'
        self.channel.message_count = 150
        self.channel.message_color = (128,255,5)
        self.health = self.max_health


    def get_speed(self):
        speed = self.moving
        speed /= self.channel.fps
        speed *= 30

        return speed
        



    

    









            



import pygame
from random import *
import toolbox as t
from balloon import Bolt
import InnPC

class Event():
    def __init__(self, server):
        self.server = server
        self.NPCs = []
    def update(self):
        pass


class BarbarianRaid():
    def __init__(self, server, num=None):
        self.server = server
        self.server.event = self

        for p in server.players:
            for b in p.get_buildings():
                if b.__class__.__name__ == 'Inn':
                    if b.NPC:
                        b.NPC.depart()
        if not num:
            self.num_barbarians = randint(4, 7)
            for p in self.server.playing:
                self.num_barbarians += 2
        else:
            self.num_barbarians = num
                
        self.spawn = choice(['North', 'East', 'South', 'West'])
        for p in self.server.players:
            p.message = 'A tribe of %s barbarians is aproaching from the %s!' % (self.num_barbarians, self.spawn)
            p.message_color = (128,0,128)
            p.message_count = 160
        
        self.barbarians = []
        self.NPCs = self.barbarians
        self.leader = BarbarianLeader(self)
        for b in range(self.num_barbarians - 1):
            Barbarian(self)

        for p in self.server.players:
            if not p.pending:
                p.Send({'action':'music_change', 'music':'barbarianraid'})

    def end(self, attacker=None):
        self.barbarians.clear()
        for p in self.server.players:
            if not p.pending:
                p.Send({'action':'music_change', 'music':'village'})
        if attacker:
            for b in attacker.channel.get_buildings():
                if b.__class__.__name__ == 'Inn':
                    NPCs = [InnPC.Retired_barbarian, InnPC.Adventurer]
                    for NPC in b.imposs:
                        NPCs.remove(NPC)
                    if NPCs:
                        b.NPC = choice(NPCs)(b)
        self.server.event = Event(self.server)

    def get_coords(self):
        coords = {'North':(3000, 400),
                'South':(3000, 3500),
                'East':(5600, 1800),
                'West':(400, 1800)
                }
        x, y = coords[self.spawn]

        x += randint(-240, 240)
        y += randint(-240, 240)

        for b in self.barbarians:
            if b.rect.colliderect(b.surf.get_rect(center=(x, y))):
                return self.get_coords()

        return x, y

    def update(self):
        try:
            for b in self.barbarians:
                b.update()
        except:
            self.end()
    

def Barbarian(event):
    eval(choice(['BarbarianArcher', 'BarbarianSwordsman', 'BarbarianSwordsman']) + '(event)')


class BarbarianLeader():
    def __init__(self, event):
        self.event = event
        self.surf = pygame.Surface((50, 50))
        self.rect = self.surf.get_rect(center=(0, 0))
        self.x, self.y = self.event.get_coords()
        self.event.barbarians.append(self)
        self.angle = 0
        self.surf = pygame.Surface((50, 50))
        self.health = 100
        self.rect = self.surf.get_rect(center=(self.x, self.y))
        self.hurt = 0
        self.shield_count = 0
        self.shield_angle = 0
        self.speed = 2
        self.resistance = 2
        for p in self.event.server.playing:
            self.resistance += 1.5

    @property
    def innerrect(self):
        return self.rect

    def explode(self, player, tnt):
        self.getHurt(player, 60, t.getAngle(tnt.x, tnt.y, self.x, self.y), 120)
        
    def update(self):
        if self.hurt:
            self.hurt -= 1
        if self.shield_count:
            self.shield_count -= 1

        

        for ob in self.event.server.obs:
            if self.rect.colliderect(ob.innerrect):
                self.getHurt(None, 8, self.angle + 180, 100)

        maxdist = 10000
        player = None
        playerdist = maxdist
        for p in self.event.server.players:
            if not p.pending and not p.character.dead:
                if t.getDist(self.x, self.y, p.character.x, p.character.y) < maxdist:
                    maxdist = t.getDist(self.x, self.y, p.character.x, p.character.y)
                    player = p
                    playerdist = maxdist

        if player == None:
            for p in self.event.server.players:
                if not p.pending:
                    p.to_send.append({'action':'draw_barbarian',
                                  'type':'leader',
                                  'hurt':self.hurt,
                                  'coords':(p.character.get_x(self), p.character.get_y(self)),
                                  'angle':self.angle,
                                  'health':self.health,
                                  'shield':(self.shield_angle if self.shield_count else None)})
            return

        self.angle = t.getAngle(self.x, self.y, player.character.x, player.character.y)

        if playerdist < 400:
            self.speed = 1
        else:
            self.speed = 2

        x, y = t.getDir(self.angle, total=self.speed)

        self.x += x
        rect = self.surf.get_rect(center=(self.x, self.y))
        for ob in self.event.server.obs + self.event.barbarians:
            if rect.colliderect(ob.innerrect) and ob != self:
                self.x -= x
        self.y += y
        rect = self.surf.get_rect(center=(self.x, self.y))
        for ob in self.event.server.obs + self.event.barbarians:
            if rect.colliderect(ob.innerrect) and ob != self:
                self.y -= y

        self.rect = self.surf.get_rect(center=(self.x, self.y))



        for p in self.event.server.players:
            if not p.pending:
                if p.character.rect.colliderect(self.rect) and not p.character.dead:
                    p.character.getHurt(10, 'The Barbarian Leader', self.angle, 50)
                    if p.character.dead:
                        p.character.gold = 0
                        p.character.food = 0
        
        for p in self.event.server.players:
            if not p.pending:
                p.to_send.append({'action':'draw_barbarian',
                                  'type':'leader',
                                  'hurt':self.hurt,
                                  'coords':(p.character.get_x(self), p.character.get_y(self)),
                                  'angle':self.angle,
                                  'health':self.health,
                                  'shield':(self.shield_angle if self.shield_count else None)})
                
    def getHurt(self, attacker, damage, angle, knockback):
        if randint(0, 2) < 2 and attacker != self:
            self.shield_angle = angle + 180
            self.shield_count = 20
            for p in self.event.server.players:
                if not p.pending:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    rect = pygame.Rect(0, 0, 1, 1)
                    rect.size = self.rect.size
                    rect.topleft = (p.character.get_x(self.rect), p.character.get_y(self.rect))
                    if screen.colliderect(rect):
                        p.to_send.append({'action':'sound','sound':'bump'})
            return ('repelled', self.shield_angle + randint(-20, 20))
        else:
            x, y = t.getDir(angle, total=(self.speed+knockback))

            self.x += x
            rect = self.surf.get_rect(center=(self.x, self.y))
            for ob in self.event.server.obs + self.event.barbarians:
                if rect.colliderect(ob.innerrect) and ob != self:
                    self.x -= x
            self.y += y
            rect = self.surf.get_rect(center=(self.x, self.y))
            for ob in self.event.server.obs + self.event.barbarians:
                if rect.colliderect(ob.innerrect) and ob != self:
                    self.y -= y

            self.hurt = 10

            self.rect = self.surf.get_rect(center=(self.x, self.y))

            self.health -= damage - self.resistance
            if self.health <= 0:
                self.event.barbarians.remove(self)
                self.event.end(attacker)
                self.event.server.event = Event(self.event.server)
                for p in self.event.server.players:
                    if not p.pending:
                        p.message = 'The Barbarian tribe has been defeated.'
                        p.message_color = (128,0,128)
                        p.message_count = 160
                try:
                    attacker.gold += 300
                    attacker.food += 300
                except:
                    pass


class BarbarianArcher():
    def __init__(self, event):
        self.event = event
        self.surf = pygame.Surface((50, 50))
        self.event.barbarians.append(self)
        self.rect = self.surf.get_rect(center=(0, 0))
        self.x, self.y = self.event.get_coords()
        self.angle = 0
        self.hurt = 0
        self.surf = pygame.Surface((50, 50))
        self.health = 100
        self.rect = self.surf.get_rect(center=(self.x, self.y))
        self.shield_count = 0
        self.shield_angle = 0
        self.gold = 50
        self.food = 50
        self.attack = 10
        self.knockback = 40
        self.balloon_speed = 17
        self.shoot_cooldown = 0
        self.speed = 2

    @property
    def innerrect(self):
        return self.rect

    def explode(self, player, tnt):
        self.getHurt(player, 60, t.getAngle(tnt.x, tnt.y, self.x, self.y), 120)
        
    def update(self):
        if self.hurt:
            self.hurt -= 1
        if self.shield_count:
            self.shield_count -= 1
        if self.shoot_cooldown:
            self.shoot_cooldown -= 1

        

        for ob in self.event.server.obs:
            if self.rect.colliderect(ob.innerrect):
                self.x = self.event.leader.x
                self.y = self.event.leader.y
                condition = True
                while condition:
                    condition = False
                    for ob in self.event.server.obs + self.event.barbarians:
                        if self.rect.colliderect(ob.innerrect) and ob != self:
                            condition = True
                    x, y = t.getDir(self.angle, total=self.speed)
                    self.x += x
                    self.y += y
                    self.rect = self.surf.get_rect(center=(self.x, self.y))

                    
            
        maxdist = 10000
        playerdist = maxdist
        player = None
        for p in self.event.server.players:
            if not p.pending and not p.character.dead:
                if t.getDist(self.x, self.y, p.character.x, p.character.y) < maxdist:
                    maxdist = t.getDist(self.x, self.y, p.character.x, p.character.y)
                    player = p
                    playerdist = maxdist

        if player == None:
            for p in self.event.server.players:
                if not p.pending:
                    p.to_send.append({'action':'draw_barbarian',
                                  'type':'archer',
                                  'hurt':self.hurt,
                                  'coords':(p.character.get_x(self), p.character.get_y(self)),
                                  'angle':self.angle,
                                  'health':self.health,
                                  'shield':(self.shield_angle if self.shield_count else None)})
            return

        self.angle = t.getAngle(self.x, self.y, player.character.x, player.character.y)

        if playerdist < 350:
            self.speed = 0
            self.shoot()
        else:
            self.speed = 2

        x, y = t.getDir(self.angle, total=self.speed)

        self.x += x
        rect = self.surf.get_rect(center=(self.x, self.y))
        for ob in self.event.server.obs + self.event.barbarians:
            if rect.colliderect(ob.innerrect) and ob != self:
                self.x -= x
        self.y += y
        rect = self.surf.get_rect(center=(self.x, self.y))
        for ob in self.event.server.obs + self.event.barbarians:
            if rect.colliderect(ob.innerrect) and ob != self:
                self.y -= y

        self.rect = self.surf.get_rect(center=(self.x, self.y))

        for item in self.event.server.bushes:
            if self.rect.colliderect(item.innerrect):
                if item.__class__.__name__ == 'SpikyBush':
                    self.getHurt(self, 4.4, self.angle + 180, 0)
                else:
                    self.getHurt(self, 4, self.angle + 180, 0)

        for p in self.event.server.players:
            if not p.pending:
                if p.character.rect.colliderect(self.rect) and not p.character.dead:
                    p.character.getHurt(10, 'a Barbarian Archer', self.angle, 50, msg='<Victim> ran into <Attacker>. Their gold and food were stolen.')
                    if p.character.dead:
                        self.gold += p.character.gold
                        self.food += p.character.food
                        p.character.gold = 0
                        p.character.food = 0

        
        for p in self.event.server.players:
            if not p.pending:
                p.to_send.append({'action':'draw_barbarian',
                                  'type':'archer',
                                  'hurt':self.hurt,
                                  'coords':(p.character.get_x(self), p.character.get_y(self)),
                                  'angle':self.angle,
                                  'health':self.health,
                                  'shield':(self.shield_angle if self.shield_count else None)})

    def getHurt(self, attacker, damage, angle, knockback):
        if randint(0, 2) == 0 and attacker != self:
            self.shield_angle = angle + 180
            self.shield_count = 20
            for p in self.event.server.players:
                if not p.pending:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    rect = pygame.Rect(0, 0, 1, 1)
                    rect.size = self.rect.size
                    rect.topleft = (p.character.get_x(self.rect), p.character.get_y(self.rect))
                    if screen.colliderect(rect):
                        p.to_send.append({'action':'sound','sound':'bump'})
            return ('repelled', self.shield_angle + randint(-20, 20))
        else:
        
            x, y = t.getDir(angle, total=(self.speed+knockback))

            self.x += x
            rect = self.surf.get_rect(center=(self.x, self.y))
            for ob in self.event.server.obs + self.event.barbarians:
                if rect.colliderect(ob.innerrect) and ob != self:
                    self.x -= x
            self.y += y
            rect = self.surf.get_rect(center=(self.x, self.y))
            for ob in self.event.server.obs + self.event.barbarians:
                if rect.colliderect(ob.innerrect) and ob != self:
                    self.y -= y
            self.hurt = 10
            self.rect = self.surf.get_rect(center=(self.x, self.y))

            self.health -= damage - 4
            if self.health <= 0:
                self.event.barbarians.remove(self)
                for p in self.event.server.players:
                    if not p.pending:
                        p.message = 'A Barbarian Archer has been defeated!'
                        p.message_color = (255,205,0)
                        p.message_count = 160
                attacker.gold += self.gold
                attacker.food += self.food

    def shoot(self):
        if not self.shoot_cooldown:
            self.shoot_cooldown = 50
            Bolt(self)


class BarbarianSwordsman():
    def __init__(self, event):
        self.event = event
        self.surf = pygame.Surface((50, 50))
        self.event.barbarians.append(self)
        self.rect = self.surf.get_rect(center=(0, 0))
        self.x, self.y = self.event.get_coords()
        self.rect = self.surf.get_rect(center=(self.x, self.y))
        self.angle = 0
        self.hurt = 0
        self.speed = 5
        self.health = 100
        self.shield_count = 0
        self.shield_angle = 0
        self.gold = 50
        self.food = 50
        self.speed = 2


    @property
    def innerrect(self):
        return self.rect

    def explode(self, player, tnt):
        self.getHurt(player, 60, t.getAngle(tnt.x, tnt.y, self.x, self.y), 120)
        
    def update(self):
        if self.hurt:
            self.hurt -= 1
        if self.shield_count:
            self.shield_count -= 1


        for ob in self.event.server.obs:
            if self.rect.colliderect(ob.innerrect):
                self.x = self.event.leader.x
                self.y = self.event.leader.y
                condition = True
                while condition:
                    condition = False
                    for ob in self.event.server.obs + self.event.barbarians:
                        if self.rect.colliderect(ob.innerrect) and ob != self:
                            condition = True
                    x, y = t.getDir(self.angle, total=self.speed)
                    self.x += x
                    self.y += y
                    self.rect = self.surf.get_rect(center=(self.x, self.y))
            
        maxdist = 10000
        playerdist = maxdist
        player = None
        for p in self.event.server.players:
            if not p.pending and not p.character.dead:
                if t.getDist(self.x, self.y, p.character.x, p.character.y) < maxdist:
                    maxdist = t.getDist(self.x, self.y, p.character.x, p.character.y)
                    player = p
                    playerdist = maxdist

        if player == None:
            for p in self.event.server.players:
                if not p.pending:
                    p.to_send.append({'action':'draw_barbarian',
                                  'type':'swordsman',
                                  'hurt':self.hurt,
                                  'coords':(p.character.get_x(self), p.character.get_y(self)),
                                  'angle':self.angle,
                                  'health':self.health,
                                  'shield':(self.shield_angle if self.shield_count else None)})
            return

        self.angle = t.getAngle(self.x, self.y, player.character.x, player.character.y)

        if playerdist < 350:
            self.speed = 5
        else:
            self.speed = 2

        x, y = t.getDir(self.angle, total=self.speed)

        self.x += x
        rect = self.surf.get_rect(center=(self.x, self.y))
        for ob in self.event.server.obs + self.event.barbarians:
            if rect.colliderect(ob.innerrect) and ob != self:
                self.x -= x
        self.y += y
        rect = self.surf.get_rect(center=(self.x, self.y))
        for ob in self.event.server.obs + self.event.barbarians:
            if rect.colliderect(ob.innerrect) and ob != self:
                self.y -= y

        self.rect = self.surf.get_rect(center=(self.x, self.y))

        for item in self.event.server.bushes:
            if self.rect.colliderect(item.innerrect):
                if item.__class__.__name__ == 'SpikyBush':
                    self.getHurt(self, 4.4, self.angle + 180, 0)
                else:
                    self.getHurt(self, 4, self.angle + 180, 0)

        for p in self.event.server.players:
            if not p.pending:
                if p.character.rect.colliderect(self.rect) and not p.character.dead:
                    p.character.getHurt(10, 'A Barbarian Swordsman', self.angle, 50, msg='<Attacker> killed <Victim> and stole their gold and food.')
                    if p.character.dead:
                        self.gold += p.character.gold
                        self.food += p.character.food
                        p.character.gold = 0
                        p.character.food = 0



        
        for p in self.event.server.players:
            if not p.pending:
                p.to_send.append({'action':'draw_barbarian',
                                  'type':'swordsman',
                                  'hurt':self.hurt,
                                  'coords':(p.character.get_x(self), p.character.get_y(self)),
                                  'angle':self.angle,
                                  'health':self.health,
                                  'shield':(self.shield_angle if self.shield_count else None)})

    def getHurt(self, attacker, damage, angle, knockback):

        if randint(0, 2) == 0 and attacker != self:
            self.shield_angle = angle + 180
            self.shield_count = 20
            for p in self.event.server.players:
                if not p.pending:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    rect = pygame.Rect(0, 0, 1, 1)
                    rect.size = self.rect.size
                    rect.topleft = (p.character.get_x(self.rect), p.character.get_y(self.rect))
                    if screen.colliderect(rect):
                        p.to_send.append({'action':'sound','sound':'bump'})
            return ('repelled', self.shield_angle + randint(-20, 20))
        else:
        
            x, y = t.getDir(angle, total=(self.speed+knockback))

            self.x += x
            rect = self.surf.get_rect(center=(self.x, self.y))
            for ob in self.event.server.obs + self.event.barbarians:
                if rect.colliderect(ob.innerrect) and ob != self:
                    self.x -= x
            self.y += y
            rect = self.surf.get_rect(center=(self.x, self.y))
            for ob in self.event.server.obs + self.event.barbarians:
                if rect.colliderect(ob.innerrect) and ob != self:
                    self.y -= y
            self.hurt = 10
            self.rect = self.surf.get_rect(center=(self.x, self.y))

            self.health -= damage - 4
            if self.health <= 0:
                self.event.barbarians.remove(self)
                for p in self.event.server.players:
                    if not p.pending:
                        p.message = 'A Barbarian Swordsman has been defeated!'
                        p.message_color = (255,205,0)
                        p.message_count = 160
                attacker.gold += self.gold
                attacker.food += self.food
            





















            




import pygame

class Animation(pygame.sprite.Sprite):
    def __init__(self, item):
        super().__init__(self.gp)
        self.frame = 0
        self.len = 3
        self.server = (item.server if hasattr(item, 'server') else item.channel.server)
        self.size = 50
        self.x = item.x
        self.y = item.y
        
    def __len__(self):
        return self.len

    def update(self):
        self.frame += 1
        if len(self) == self.frame:
            self.kill()
        else:
            for p in self.server.players:
                if not p.pending:
                    screen = pygame.Rect(0, 0, 1000, 650)
                    rect = pygame.Rect(0, 0, self.size, self.size)
                    rect.center = (p.character.get_x(self), p.character.get_y(self))
                    if screen.colliderect(rect):
                        p.to_send.append({'action':'animation',
                                    'coords':(p.character.get_x(self), p.character.get_y(self)),
                                    'name':self.__class__.__name__,
                                    'frame':self.frame})


class Splash(Animation):
    def __init__(self, balloon):
        super().__init__(balloon)
        self.size = 30

class Explosion(Animation):
    def __init__(self, item):
        super().__init__(item)
        self.size = 64

class BAM(Animation):
    def __init__(self, TNT):
        super().__init__(TNT)
        self.size = 360
        self.len = 6



import pygame
import math
import socket

def rotate(image, rect, angle):
    """
    rotate returns the rotated version of image and the rotated image's rectangle.
    """
    # Get a new image that is the original image but rotated
    new_image = pygame.transform.rotate(image, angle)
    # Get a new rect with the center of the old rect
    new_rect = new_image.get_rect(center=rect.center)
    return new_image, new_rect

def getDist(x1, y1, x2, y2):
    """
    getDist returns the distance between (x1, y1) and (x2, y2).
    """
    
    x = abs(x1-x2)
    y = abs(y1-y2)
    return math.sqrt(x*x+y*y)


def getAngle(x1, y1, x2, y2):
    """
    getAnlge returns the angle (x1, y1) should be at if it is facing (x2, y2).
    """
    
    x_difference = x2-x1
    y_differnece = y2-y1
    return math.degrees(math.atan2(-y_differnece, x_difference))

def getDir(angle, total=1):
    """
    returns (x, y) where x and y are the distance moved if the unit is moving at the angle.
    """
    
    angle_rads = math.radians(angle)
    x_move = math.cos(angle_rads) * total
    y_move = -(math.sin(angle_rads) * total)
    return x_move, y_move


def centerXY(thing, screen):
    """
    centeringCoords returns the coords that will put thing in the center of the screen.
    """
    new_x = screen.get_width()/2 - thing.get_width()/2
    new_y = screen.get_height()/2 - thing.get_height()/2
    return new_x, new_y


class keyDownListener():
    """
    keyDownListener keeps track of one key and says when it gets pressed down.
    self.down will be True once every time the key is pressed and False otherwise.
    """
    def __init__(self):
        self.down = False
        self.is_up = False
        
    def update(self, key_pressed):
        if not key_pressed:
            self.is_up = True
            self.down = False
        else:
            if self.is_up:
                self.down = True
            else:
                self.down = False
            self.is_up = False


def getMyIP():
    """
    getMyIP returns the IP address on the computer you run it on.
    This may not work properly if you're not connected to the internet.
    (If this code seems super complicated, don't worry.. I don't understand it either  -Andy)
    """
    return str((([ip for ip in socket.gethostbyname_ex(socket.gethostname())[2] if not ip.startswith("127.")]
             or [[(s.connect(("8.8.8.8", 53)), s.getsockname()[0], s.close()) for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]][0][1]])
            + ["no IP found"])[0])




def copy(theList):
    return theList[:]





try:
    from PodSixNet.Connection import ConnectionListener, connection
    import toolbox
except:
    from src.PodSixNet.Connection import ConnectionListener, connection
    from src import toolbox
import os
import zipfile
import sys

import pymsgbox

print_output = True
ip = 'NOT_INITIALIZED'
FINISHED = False

def init(IP, printOutput=False):
    global print_output
    print_output = printOutput
    global ip
    ip = IP



class DownloadClient(ConnectionListener):

    def __init__(self, host, port):
        self.last_perc = 0
        self.num = 0
        self.amount = 0
        self.namelist = []
        self.running = True
        for foldername, subfolders, filenames in os.walk('../'):
            self.namelist.append(foldername)
            self.namelist.extend(filenames)
        
        ConnectionListener.__init__(self)
        self.Connect((host, port))
        
        

    def Network_open(self, data):

        self.file = open('../run/downloads/' + data['version'] + '.zip', 'wb')
        self.version = data['version']
        

    def update(self):
        connection.Pump()
        self.Pump()

    def ShutDown(self):
        self.connected = False
        connection.Close()

    def Network_downloaded(self, data):
        perc = data['num'] / data['amount'] * 100
        global print_output
        if print_output and perc != self.last_perc:
            print('Downloading - ' + str(round(perc)) + '%', data['amount'])
        self.last_perc = perc
        self.file.write(eval(data['bytes']))

        if data['num'] == data['amount']:
            self.file.close()
            if print_output:
                print('Downloaded VillageWars version ' + self.version + '!')
            self.running = False
        else:
            self.file.write(b'\n')

        self.num = data['num']
        self.amount = data['amount']
            

    def Network_connected(self, data):
        print('Connection succesful!')
        print('Updating VillageWars...')
        
        self.connected = True
        connection.Send({'action':'init', 'status':'DOWNLOAD', 'namelist':self.namelist})
        
        
        

        

    def Network_error(self, data):
        """
        Network_error runs when there is a server error
        """
        print('error:', data['error'][1])
        self.ShutDown()

    def Network_disconnected(self, data):
        """
        Network_disconnected runs when you disconnect from the server
        """
        self.ShutDown()

def main():
    global print_output, FINISHED, ip
    if print_output:
        input('This is the VillageWars updating and downloading system! Press enter to start.')
        if ip == 'NOT_INITIALIZED':
            ip = input('Enter the host\'s IP address: ')
        print('Connecting to host server...')



    port = 5555
    global client
    try:
        client = DownloadClient(ip, port)

    
        while client.running:
            client.update()
        print('Adding extra files...')

        file = zipfile.ZipFile(f'../run/downloads/{client.version}.zip', 'a')
        for name in client.namelist:
            if name.endswith('.mp3') or name.endswith('.wav'):
                file.write('../assets/sfx/' + name)
        file.close()

        if print_output:
            print('Extracting...')
            
        file = zipfile.ZipFile(f'../run/downloads/{client.version}.zip', 'r')

        os.makedirs('../../%s' % (client.version), exist_ok=True)
        file.extractall('../../%s' % (client.version))
        if print_output:
            print('Extraction Successful!')
            print('Finalizing...')
        file.close()
        if print_output:
            input(f'Downloaded VillageWars Version {client.version} Successfully, press enter to exit.')
        FINISHED = True
    except OSError as exc:
        answ = pymsgbox.confirm('An error occured while updating. Click OK to restart.')
        if answ == 'OK':
            main()
        else:
            
            FINISHED = 'failed'

if __name__ == '__main__':
    main()


def perc():
    global client
    try:
        return client.last_perc
    except NameError:
        return 0

def amount():
    global client
    try:
        return client.amount
    except NameError:
        return 1

def num():
    global client
    try:
        return client.num
    except NameError:
        return 0



import pygame as p
import toolbox

class LobbyAvatar():
    def __init__(self, coords):
        self.coords = coords
        self.ready = False

        self.ready_key = toolbox.keyDownListener()

    def HandleInput(self, keys):
        self.ready_key.update(keys[p.K_SPACE])
        
        if self.ready_key.down:
            self.ready = not self.ready
